@@ -34,6 +34,11 @@ import org.apache.uima.internal.util.rb_trees.RedBlackTree;
 /**
  * Feature structure implementation.
  * 
+ * This is the common super class of all Feature Structures
+ * 
+ *   including the JCAS (derived from TOP)
+ *   and non JCas FSs
+ * 
  * 
  * @version $Revision: 1.6 $
  */
@@ -68,7 +73,8 @@ public abstract class FeatureStructureImpl implements FeatureStructure, Cloneabl
 					new String[] { feat.getName(), feat.getRange().getName(), fs.getType().getName() });
 			throw e;
 		}
-		this.getCASImpl().setFeatureValue(this.getAddress(), featCode, valueAddr);
+		// keys are not fsRefs
+		this.getCASImpl().setFeatureValueNoIndexCorruptionCheck(this.getAddress(), featCode, valueAddr);
 	}
 
 	private final void setNullValue(int featCode, int rangeType) {
@@ -78,7 +84,8 @@ public abstract class FeatureStructureImpl implements FeatureStructure, Cloneabl
 					new String[] { this.getCASImpl().getTypeSystemImpl().ll_getFeatureForCode(featCode).getName() });
 			throw e;
 		}
-		this.getCASImpl().setFeatureValue(this.getAddress(), featCode, CASImpl.NULL);
+		// a null fsref is never an index key
+		this.getCASImpl().setFeatureValueNoIndexCorruptionCheck(this.getAddress(), featCode, CASImpl.NULL);
 	}
 
 	public void setIntValue(Feature feat, int val) {
@@ -158,7 +165,6 @@ public abstract class FeatureStructureImpl implements FeatureStructure, Cloneabl
 			throwIllegalRangeExc(feat, ts.getType(CAS.TYPE_NAME_BOOLEAN));
 		}
 		final int featCode = ((FeatureImpl) feat).getCode();
-
 		this.getCASImpl().setFeatureValue(this.getAddress(), featCode, b);
 
 	}
@@ -390,7 +396,7 @@ public abstract class FeatureStructureImpl implements FeatureStructure, Cloneabl
 
 		boolean addReference(int ref) {
 			if (this.tree.containsKey(ref)) {
-				String refName = (String) this.tree.get(ref);
+				String refName = this.tree.get(ref);
 				if (refName == null) {
 					refName = refNamePrefix + Integer.toString(this.count);
 					++this.count;
@@ -403,7 +409,7 @@ public abstract class FeatureStructureImpl implements FeatureStructure, Cloneabl
 		}
 
 		String getLabel(int ref) {
-			return (String) this.tree.get(ref);
+			return this.tree.get(ref);
 		}
 
 		int printInfo(int ref) {
@@ -482,7 +488,7 @@ public abstract class FeatureStructureImpl implements FeatureStructure, Cloneabl
 		if (s != null) {
 			buf.append(" \"" + s + "\"");
 		}
-		buf.append("\n");
+		buf.append('\n');
 		CommonAuxArrayFSImpl arrayFS = null;
 		LowLevelTypeSystem llts = this.getCASImpl().ll_getTypeSystem();
 		final int typeClass = this.getCASImpl().ll_getTypeClass(llts.ll_getCodeForType(this.getType()));
@@ -588,7 +594,7 @@ public abstract class FeatureStructureImpl implements FeatureStructure, Cloneabl
 		String stringVal;
 		for (int i = 0; i < feats.size(); i++) {
 			StringUtils.printSpaces(indent, buf);
-			feat = (Feature) feats.get(i);
+			feat = feats.get(i);
 			buf.append(feat.getShortName() + ": ");
 			approp = feat.getRange();
 			// System.out.println("Range type: " + approp);
