@@ -215,13 +215,15 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
   // set up lazily
   TypeInfo[] typeInfoArray;
 
-  boolean areBuiltInTypesSetup = false;
+  // saw evidence that in some cases the setup is called on the same instance on two threads
+  // must be volatile to force the right memory barriers
+  volatile boolean areBuiltInTypesSetup = false;
 
-  TypeImpl intType;
+  public TypeImpl intType;
 
-  TypeImpl stringType;
+  public TypeImpl stringType;
 
-  TypeImpl floatType;
+  public TypeImpl floatType;
 
   TypeImpl arrayBaseType;
 
@@ -233,7 +235,8 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
 
   TypeImpl fsArrayType;
 
-  TypeImpl sofaType;
+  // needed for CasCopier
+  public TypeImpl sofaType;
 
   TypeImpl annotType;
 
@@ -249,23 +252,23 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
 
   FeatureImpl sofaNum;
 
-  TypeImpl byteType;
+  public TypeImpl byteType;
 
   TypeImpl byteArrayType;
 
-  TypeImpl booleanType;
+  public TypeImpl booleanType;
 
   TypeImpl booleanArrayType;
 
-  TypeImpl shortType;
+  public TypeImpl shortType;
 
   TypeImpl shortArrayType;
 
-  TypeImpl longType;
+  public TypeImpl longType;
 
   TypeImpl longArrayType;
 
-  TypeImpl doubleType;
+  public TypeImpl doubleType;
 
   TypeImpl doubleArrayType;
 
@@ -278,13 +281,13 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
 
   int arrayBaseTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
 
-  int intArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public int intArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
 
-  int floatArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public int floatArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
 
-  int stringArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public int stringArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
 
-  int fsArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public int fsArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
 
   int sofaTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
 
@@ -302,39 +305,42 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
 
   int doubleTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
 
-  int byteArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public int byteArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
 
-  int booleanArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public int booleanArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
 
-  int shortArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public int shortArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
 
-  int longArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public int longArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
 
-  int doubleArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public int doubleArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
 
-  public int sofaNumFeatCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;  // ref from another pkg
+  public int sofaNumFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;  // ref from another pkg
 
-  int sofaIdFeatCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public int sofaIdFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
+  
+  public int sofaStringFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
 
-  int sofaMimeFeatCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  int sofaMimeFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
 
-  int sofaUriFeatCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  int sofaUriFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
 
-  int sofaArrayFeatCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  int sofaArrayFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
 
-  public int annotSofaFeatCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE; // ref from another pkg
+  public int annotSofaFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE; // ref from another pkg
 
-  int startFeatCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public int startFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
 
-  int endFeatCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public int endFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
 
-  int langFeatCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  int langFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
 
   /**
    * Default constructor.
    * 
    * @deprecated Use 0 arg constructor. Type Systems are shared by many CASes, and can't point to
    *             one. Change also your possible calls to ts.commit() - see comment on that method.
+   * @param cas -
    */
   @Deprecated
   public TypeSystemImpl(CASImpl cas) {
@@ -383,63 +389,63 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
   final void initTypeVariables() {
     // Type objects.
     // this.ts.topType = (TypeImpl) this.ts.getTopType(); // never used
-    this.intType = (TypeImpl) getType(CAS.TYPE_NAME_INTEGER);
-    this.stringType = (TypeImpl) getType(CAS.TYPE_NAME_STRING);
-    this.floatType = (TypeImpl) getType(CAS.TYPE_NAME_FLOAT);
-    this.arrayBaseType = (TypeImpl) getType(CAS.TYPE_NAME_ARRAY_BASE);
-    this.intArrayType = (TypeImpl) getType(CAS.TYPE_NAME_INTEGER_ARRAY);
-    this.floatArrayType = (TypeImpl) getType(CAS.TYPE_NAME_FLOAT_ARRAY);
-    this.stringArrayType = (TypeImpl) getType(CAS.TYPE_NAME_STRING_ARRAY);
-    this.fsArrayType = (TypeImpl) getType(CAS.TYPE_NAME_FS_ARRAY);
-    this.sofaType = (TypeImpl) getType(CAS.TYPE_NAME_SOFA);
-    this.annotType = (TypeImpl) getType(CAS.TYPE_NAME_ANNOTATION);
+//    this.intType = (TypeImpl) getType(CAS.TYPE_NAME_INTEGER);
+//    this.stringType = (TypeImpl) getType(CAS.TYPE_NAME_STRING);
+//    this.floatType = (TypeImpl) getType(CAS.TYPE_NAME_FLOAT);
+//    this.arrayBaseType = (TypeImpl) getType(CAS.TYPE_NAME_ARRAY_BASE);
+//    this.intArrayType = (TypeImpl) getType(CAS.TYPE_NAME_INTEGER_ARRAY);
+//    this.floatArrayType = (TypeImpl) getType(CAS.TYPE_NAME_FLOAT_ARRAY);
+//    this.stringArrayType = (TypeImpl) getType(CAS.TYPE_NAME_STRING_ARRAY);
+//    this.fsArrayType = (TypeImpl) getType(CAS.TYPE_NAME_FS_ARRAY);
+//    this.sofaType = (TypeImpl) getType(CAS.TYPE_NAME_SOFA);
+//    this.annotType = (TypeImpl) getType(CAS.TYPE_NAME_ANNOTATION);
     this.sofaNum = (FeatureImpl) getFeatureByFullName(CAS.FEATURE_FULL_NAME_SOFANUM);
     this.annotBaseType = (TypeImpl) getType(CAS.TYPE_NAME_ANNOTATION_BASE);
     this.startFeat = (FeatureImpl) getFeatureByFullName(CAS.FEATURE_FULL_NAME_BEGIN);
     this.endFeat = (FeatureImpl) getFeatureByFullName(CAS.FEATURE_FULL_NAME_END);
     this.langFeat = (FeatureImpl) getFeatureByFullName(CAS.FEATURE_FULL_NAME_LANGUAGE);
-    this.docType = (TypeImpl) getType(CAS.TYPE_NAME_DOCUMENT_ANNOTATION);
+//    this.docType = (TypeImpl) getType(CAS.TYPE_NAME_DOCUMENT_ANNOTATION);
 
-    this.byteType = (TypeImpl) getType(CAS.TYPE_NAME_BYTE);
-    this.byteArrayType = (TypeImpl) getType(CAS.TYPE_NAME_BYTE_ARRAY);
-    this.booleanType = (TypeImpl) getType(CAS.TYPE_NAME_BOOLEAN);
-    this.booleanArrayType = (TypeImpl) getType(CAS.TYPE_NAME_BOOLEAN_ARRAY);
-    this.shortType = (TypeImpl) getType(CAS.TYPE_NAME_SHORT);
-    this.shortArrayType = (TypeImpl) getType(CAS.TYPE_NAME_SHORT_ARRAY);
-    this.longType = (TypeImpl) getType(CAS.TYPE_NAME_LONG);
-    this.longArrayType = (TypeImpl) getType(CAS.TYPE_NAME_LONG_ARRAY);
-    this.doubleType = (TypeImpl) getType(CAS.TYPE_NAME_DOUBLE);
-    this.doubleArrayType = (TypeImpl) getType(CAS.TYPE_NAME_DOUBLE_ARRAY);
+//    this.byteType = (TypeImpl) getType(CAS.TYPE_NAME_BYTE);
+//    this.byteArrayType = (TypeImpl) getType(CAS.TYPE_NAME_BYTE_ARRAY);
+//    this.booleanType = (TypeImpl) getType(CAS.TYPE_NAME_BOOLEAN);
+//    this.booleanArrayType = (TypeImpl) getType(CAS.TYPE_NAME_BOOLEAN_ARRAY);
+//    this.shortType = (TypeImpl) getType(CAS.TYPE_NAME_SHORT);
+//    this.shortArrayType = (TypeImpl) getType(CAS.TYPE_NAME_SHORT_ARRAY);
+//    this.longType = (TypeImpl) getType(CAS.TYPE_NAME_LONG);
+//    this.longArrayType = (TypeImpl) getType(CAS.TYPE_NAME_LONG_ARRAY);
+//    this.doubleType = (TypeImpl) getType(CAS.TYPE_NAME_DOUBLE);
+//    this.doubleArrayType = (TypeImpl) getType(CAS.TYPE_NAME_DOUBLE_ARRAY);
 
     // Type codes.
     initTypeCodeVars();
   }
 
   private final void initTypeCodeVars() {
-    this.intTypeCode = this.intType.getCode();
-    this.stringTypeCode = this.stringType.getCode();
-    this.floatTypeCode = this.floatType.getCode();
+//    this.intTypeCode = this.intType.getCode();
+//    this.stringTypeCode = this.stringType.getCode();
+//    this.floatTypeCode = this.floatType.getCode();
     // this.arrayBaseTypeCode = arrayBaseType.getCode();
-    this.intArrayTypeCode = this.intArrayType.getCode();
-    this.floatArrayTypeCode = this.floatArrayType.getCode();
-    this.stringArrayTypeCode = this.stringArrayType.getCode();
-    this.fsArrayTypeCode = this.fsArrayType.getCode();
-    this.sofaTypeCode = this.sofaType.getCode();
-    this.annotTypeCode = this.annotType.getCode();
-    this.annotBaseTypeCode = this.annotBaseType.getCode();
+//    this.intArrayTypeCode = this.intArrayType.getCode();
+//    this.floatArrayTypeCode = this.floatArrayType.getCode();
+//    this.stringArrayTypeCode = this.stringArrayType.getCode();
+//    this.fsArrayTypeCode = this.fsArrayType.getCode();
+//    this.sofaTypeCode = this.sofaType.getCode();
+//    this.annotTypeCode = this.annotType.getCode();
+//    this.annotBaseTypeCode = this.annotBaseType.getCode();
 
-    this.byteArrayTypeCode = this.byteArrayType.getCode();
-    this.byteTypeCode = this.byteType.getCode();
-    this.booleanTypeCode = this.booleanType.getCode();
-    this.booleanArrayTypeCode = this.booleanArrayType.getCode();
-    this.shortTypeCode = this.shortType.getCode();
-    this.shortArrayTypeCode = this.shortArrayType.getCode();
-    this.longTypeCode = this.longType.getCode();
-    this.longArrayTypeCode = this.longArrayType.getCode();
-    this.doubleTypeCode = this.doubleType.getCode();
-    this.doubleArrayTypeCode = this.doubleArrayType.getCode();
+//    this.byteArrayTypeCode = this.byteArrayType.getCode();
+//    this.byteTypeCode = this.byteType.getCode();
+//    this.booleanTypeCode = this.booleanType.getCode();
+//    this.booleanArrayTypeCode = this.booleanArrayType.getCode();
+//    this.shortTypeCode = this.shortType.getCode();
+//    this.shortArrayTypeCode = this.shortArrayType.getCode();
+//    this.longTypeCode = this.longType.getCode();
+//    this.longArrayTypeCode = this.longArrayType.getCode();
+//    this.doubleTypeCode = this.doubleType.getCode();
+//    this.doubleArrayTypeCode = this.doubleArrayType.getCode();
 
-    this.arrayBaseTypeCode = this.arrayBaseType.getCode();
+//    this.arrayBaseTypeCode = this.arrayBaseType.getCode();
 
     final Type sofaT = this.sofaType;
     this.sofaNumFeatCode = ll_getCodeForFeature(sofaT
@@ -460,6 +466,8 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
         .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_END));
     this.langFeatCode = ll_getCodeForFeature(this.docType
         .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_LANGUAGE));
+    this.sofaStringFeatCode = ll_getCodeForFeature(sofaT
+        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_SOFASTRING));
   }
 
   // Some implementation helpers for users of the type system.
@@ -513,6 +521,9 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
 
   /**
    * Check if feature is appropriate for type (i.e., type is subsumed by domain type of feature).
+   * @param type -
+   * @param feat -
+   * @return true if feature is appropriate for type (i.e., type is subsumed by domain type of feature).
    */
   public boolean isApprop(int type, int feat) {
     return subsumes(intro(feat), type);
@@ -850,6 +861,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
 
   /**
    * Get the overall number of features defined in the type system.
+   * @return -
    */
   public int getNumberOfFeatures() {
     if (this.isCommitted()) {
@@ -860,6 +872,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
 
   /**
    * Get the overall number of types defined in the type system.
+   * @return -
    */
   public int getNumberOfTypes() {
     if (this.isCommitted()) {
@@ -869,7 +882,9 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
   }
 
   /**
-   * Get the domain type for a feature.
+   * 
+   * @param feat -
+   * @return the domain type for a feature.
    */
   public int intro(int feat) {
     return this.intro.get(feat);
@@ -877,6 +892,8 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
 
   /**
    * Get the range type for a feature.
+   * @param feat -
+   * @return -
    */
   public int range(int feat) {
     return this.featRange.get(feat);
@@ -899,6 +916,11 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
 
   /**
    * Add a new feature to the type system.
+   * @param shortName -
+   * @param domain -
+   * @param range -
+   * @param multiRefsAllowed - 
+   * @return -
    */
   int addFeature(String shortName, int domain, int range, boolean multiRefsAllowed) {
     // Since we just looked up the domain in the symbol table, we know it
@@ -956,6 +978,8 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
 
   /**
    * Add a top type to the (empty) type system.
+   * @param name -
+   * @return -
    */
   public Type addTopType(String name) {
     final int code = this.addTopTypeInternal(name);
@@ -986,6 +1010,9 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
 
   /**
    * Check if the first argument subsumes the second
+   * @param superType -
+   * @param type -
+   * @return true if first argument subsumes the second
    */
   public boolean subsumes(int superType, int type) {
     return this.ll_subsumes(superType, type);
@@ -1084,7 +1111,9 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
 
   /**
    * This writes out the type hierarchy in a human-readable form.
+   *
    */
+  @Override
   public String toString() {
     // This code is maximally readable, not maximally efficient.
     StringBuffer buf = new StringBuffer();
@@ -1111,7 +1140,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
    * @see org.apache.uima.cas.admin.TypeSystemMgr#commit()
    */
   public void commit() {
-    if (this.locked == true) {
+    if (this.locked) {
       return; // might be called multiple times, but only need to do once
     }
     this.locked = true;
@@ -1136,13 +1165,29 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
   public boolean isCommitted() {
     return this.locked;
   }
+  
+  /**
+   * @param typeCode for a type
+   * @return true if type is AnnotationBase or a subtype of it
+   */
+  public boolean isAnnotationBaseOrSubtype(int typeCode) {
+    return subsumes(annotBaseTypeCode, typeCode);
+  }
+  
+  /**
+   * @param typeCode for a type
+   * @return true if type is Annotation or a subtype of it
+   */
+  public boolean isAnnotationOrSubtype(int typeCode) {
+    return subsumes(annotTypeCode, typeCode);
+  }  
 
   // dangerous, and not needed, not in any interface
 //  public void setCommitted(boolean b) {
 //    this.locked = b;
 //  }
 
-  /**
+  /*
    * @deprecated
    */
   @Deprecated
@@ -1443,6 +1488,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     }
     return this.stringSets.get(this.stringSetMap.get(typeCode));
   }
+  
 
   /**
    * Each instance holds info needed in binary serialization
@@ -1623,4 +1669,33 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     return m;
   }
   
+//  /**
+//   * @param otherTs type system to compare to this one
+//   * @return true if one or more identically named features have differently named ranges
+//   */
+//  public boolean isRangeCheckNeeded(TypeSystemImpl otherTs) {
+//    if (this == otherTs) {
+//      return false;
+//    }
+//    final List<Feature> smallerList;
+//    if (features.size() > otherTs.features.size()) {
+//      smallerList = otherTs.features;
+//      otherTs = this; 
+//    } else {
+//      smallerList = features;
+//    }
+//    
+//    for (int i = smallerList.size() - 1; i > 0; i--) { // position 0 is null
+//      final Feature f1 = smallerList.get(i);
+//      final Feature f2 = otherTs.getFeatureByFullName(f1.getName());
+//      if (f2 == null) {
+//        continue;
+//      }
+//      if (!f1.getRange().getName().equals(f2.getRange().getName())) {
+//        return true;
+//      }
+//    }
+//    return false;
+//  }
+//  
 }
