@@ -31,6 +31,7 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
+import java.util.Map.Entry;
 import java.util.NoSuchElementException;
 import java.util.TreeMap;
 
@@ -48,7 +49,9 @@ import org.apache.uima.cas.Type;
 import org.apache.uima.cas.TypeSystem;
 import org.apache.uima.cas.impl.XmiSerializationSharedData.OotsElementData;
 import org.apache.uima.internal.util.I18nUtil;
+import org.apache.uima.internal.util.IntListIterator;
 import org.apache.uima.internal.util.IntVector;
+import org.apache.uima.internal.util.PositiveIntSet;
 import org.apache.uima.internal.util.XmlAttribute;
 import org.apache.uima.internal.util.XmlElementName;
 import org.apache.uima.internal.util.XmlElementNameAndContents;
@@ -68,6 +71,9 @@ import org.xml.sax.helpers.XMLReaderFactory;
  */
 
 public class XmiCasDeserializer {
+  
+  private final static boolean IS_NEW_FS = true;
+  private final static boolean IS_EXISTING_FS = false;
 
   private class XmiCasDeserializerHandler extends DefaultHandler {
     // ///////////////////////////////////////////////////////////////////////
@@ -216,7 +222,16 @@ public class XmiCasDeserializer {
     //caught and reported before any updates are made to the CAS being filled.
     
     boolean disallowedViewMemberEncountered;
-    
+
+    /**
+     * a list by view of FSs to be added to the indexes
+     */
+    final private DeferredIndexUpdates toBeAdded = new DeferredIndexUpdates();
+    /**
+     * a list by view of FSs to be removed from the indexes
+     */
+    final private DeferredIndexUpdates toBeRemoved = new DeferredIndexUpdates();
+
     /**
      * Creates a SAX handler used for deserializing an XMI CAS.
      * @param aCAS CAS to deserialize into
@@ -253,7 +268,7 @@ public class XmiCasDeserializer {
         //new Sofas start at 2
         this.nextSofaNum = 2;
       } else {
-        this.nextSofaNum = ((CASImpl)this.casBeingFilled).getBaseSofaCount() + 1; 
+        this.nextSofaNum = this.casBeingFilled.getBaseSofaCount() + 1; 
       }
       this.deserializedFsAddrs = new IntVector();
       this.fsListNodesFromMultivaluedProperties = new IntVector();
@@ -267,7 +282,7 @@ public class XmiCasDeserializer {
       //a mergePoint was specified)
       FSIterator<SofaFS> sofaIter = this.casBeingFilled.getSofaIterator();
       while(sofaIter.hasNext()) {
-        SofaFS sofa = (SofaFS)sofaIter.next();
+        SofaFS sofa = sofaIter.next();
         if (sofa.getSofaRef() == 1) {
           casBeingFilled.registerInitialSofa();
         } else {
@@ -354,19 +369,19 @@ public class XmiCasDeserializer {
             if (id != null) {
               int idInt = Integer.parseInt(id);
               if (idInt > 0 && !this.isNewFS(idInt)) { //preexisting FS
-            	if (this.allowPreexistingFS == AllowPreexistingFS.ignore) { //skip elements whose ID is <= mergePoint
-            		this.state = IGNORING_XMI_ELEMENTS_STATE;
-            		this.ignoreDepth++;
-            		return;
-            	} else if (this.allowPreexistingFS == AllowPreexistingFS.disallow) { //fail 
-            		CASRuntimeException e = new CASRuntimeException(
-          		          CASRuntimeException.DELTA_CAS_PREEXISTING_FS_DISALLOWED,
-          		          new String[] {ID_ATTR_NAME + "=" + id,
-          		        		        nameSpaceURI,
-          		        		        localName,
-          		        		        qualifiedName});
-          		    throw e;
-            	}
+              	if (this.allowPreexistingFS == AllowPreexistingFS.ignore) { //skip elements whose ID is <= mergePoint
+              		this.state = IGNORING_XMI_ELEMENTS_STATE;
+              		this.ignoreDepth++;
+              		return;
+              	} else if (this.allowPreexistingFS == AllowPreexistingFS.disallow) { //fail 
+              		CASRuntimeException e = new CASRuntimeException(
+            		          CASRuntimeException.DELTA_CAS_PREEXISTING_FS_DISALLOWED,
+            		          new String[] {ID_ATTR_NAME + "=" + id,
+            		        		        nameSpaceURI,
+            		        		        localName,
+            		        		        qualifiedName});
+            		    throw e;
+            	  }
               }
             }
           }
@@ -376,7 +391,7 @@ public class XmiCasDeserializer {
             int colonIndex = qualifiedName.indexOf(':');
             if (colonIndex != -1) {
               String prefix = qualifiedName.substring(0, colonIndex);
-              nameSpaceURI = (String) nsPrefixToUriMap.get(prefix);
+              nameSpaceURI = nsPrefixToUriMap.get(prefix);
               if (nameSpaceURI == null) {
                 // unbound namespace. Rather than failing, just assume a reasonable default.
                 nameSpaceURI = "http:///" + prefix + ".ecore";
@@ -440,7 +455,14 @@ public class XmiCasDeserializer {
       }
     }
 
-    // Create a new FS.
+    /**
+     * Read one FS, create a new FS or update an existing one
+     * @param nameSpaceURI -
+     * @param localName -
+     * @param qualifiedName -
+     * @param attrs -
+     * @throws SAXException -
+     */
     private void readFS(String nameSpaceURI, String localName, String qualifiedName, 
             Attributes attrs) throws SAXException {
       String typeName = xmiElementName2uimaTypeName(nameSpaceURI, localName);
@@ -469,8 +491,10 @@ public class XmiCasDeserializer {
         }
         return;
       } else if (casBeingFilled.isArrayType(currentType)) {
+        
         // store ID and array values (if specified as attribute).
         // we will actually create the array later, in endElement.
+        
         String idStr = attrs.getValue(ID_ATTR_NAME);
         currentArrayId = idStr == null ? -1 : Integer.parseInt(idStr);
         String elements = attrs.getValue("elements");
@@ -489,26 +513,34 @@ public class XmiCasDeserializer {
           }
         }
       } else {
-    	String idStr = attrs.getValue(ID_ATTR_NAME);
-    	int xmiId = idStr == null ? -1 : Integer.parseInt(idStr);
-
-    	if (isNewFS(xmiId)) {  //new FS so create it.  
-    		final int addr = casBeingFilled.ll_createFS(currentType.getCode());
-        	readFS(addr, attrs);
-    	} else {  //preexisting
-    		if (this.allowPreexistingFS == AllowPreexistingFS.disallow) {
-    		  CASRuntimeException e = new CASRuntimeException(
-        		          CASRuntimeException.DELTA_CAS_PREEXISTING_FS_DISALLOWED,
-        		          new String[] {ID_ATTR_NAME + "=" + idStr,
-        		        		        nameSpaceURI,
-        		        		        localName,
-        		        		        qualifiedName});
-    		  throw e;
-    		} else if (this.allowPreexistingFS == AllowPreexistingFS.allow) { //get the FS 
-    		  final int addr = getFsAddrForXmiId(xmiId);
-       	  	  readFS(addr,attrs);
-    		} // otherwise ignore
-    	}
+        
+        // not an array type
+        
+      	String idStr = attrs.getValue(ID_ATTR_NAME);
+      	int xmiId = idStr == null ? -1 : Integer.parseInt(idStr);
+  
+      	if (isNewFS(xmiId)) {  //new FS so create it.  
+      		final int addr = casBeingFilled.ll_createFS(currentType.getCode());
+          readFS(addr, attrs, IS_NEW_FS);
+      	} else {  //preexisting
+      		if (this.allowPreexistingFS == AllowPreexistingFS.disallow) {
+      		  CASRuntimeException e = new CASRuntimeException(
+          		          CASRuntimeException.DELTA_CAS_PREEXISTING_FS_DISALLOWED,
+          		          new String[] {ID_ATTR_NAME + "=" + idStr,
+          		        		        nameSpaceURI,
+          		        		        localName,
+          		        		        qualifiedName});
+      		  throw e;
+      		} else if (this.allowPreexistingFS == AllowPreexistingFS.allow) { //get the FS 
+      		  final int addr = getFsAddrForXmiId(xmiId);
+      		  // remove from indexes, and remember if was there, per view
+      		  //   (might be indexed in some views, not in others)
+//      		  FSsTobeReindexed modifySafely = casBeingFilled.modifySafely()
+//       		  casBeingFilled.removeFromCorruptableIndexAnyView(addr, toBeAddedBack);
+            
+       	    readFS(addr, attrs, IS_EXISTING_FS);
+      		} // otherwise ignore
+      	}
       }
     }
 
@@ -521,30 +553,22 @@ public class XmiCasDeserializer {
      * @param membersString
      *          whitespace-separated string of FS addresses. Each FS is to be added to the specified
      *          sofa's index repository
+     *          The adding takes place after FSs are finalized, to enable checking the sofa refs are OK
+     *          https://issues.apache.org/jira/browse/UIMA-4099
      */
     private void processView(String sofa, String membersString) throws SAXParseException {
       // TODO: this requires View to come AFTER all of its members
       if (membersString != null) {
-        // a view with no Sofa will be added to the 1st, _InitialView, index
-        int sofaNum = 1;
-        boolean newview = false;
-        if (sofa != null) {
-          // translate sofa's xmi:id into its sofanum
-          int sofaXmiId = Integer.parseInt(sofa);
-          newview = isNewFS(sofaXmiId);
-          int sofaAddr;
-          try {
-            sofaAddr = getFsAddrForXmiId(sofaXmiId);
-          } catch (NoSuchElementException e) {
-            throw createException(XCASParsingException.UNKNOWN_ID, Integer.toString(sofaXmiId));
-          }
-          sofaNum = casBeingFilled.getFeatureValue(sofaAddr, sofaNumFeatCode);
-        }
-        FSIndexRepositoryImpl indexRep = (FSIndexRepositoryImpl) indexRepositories.get(sofaNum);
-
+        final int sofaXmiId = (sofa == null) ? 1 : Integer.parseInt(sofa);
+        FSIndexRepositoryImpl indexRep = getIndexRepo(sofa, sofaXmiId);
+        final boolean newview = (sofa == null) ? false : isNewFS(sofaXmiId);
+        
+        final PositiveIntSet todo = toBeAdded.getTodos(indexRep);
+        
         // TODO: optimize by going straight to int[] without going through
         // intermediate String[]?
         String[] members = parseArray(membersString);
+        
         for (int i = 0; i < members.length; i++) {
           int id = Integer.parseInt(members[i]);
           // special handling for merge operations ...
@@ -561,11 +585,10 @@ public class XmiCasDeserializer {
             }
           }
           // have to map each ID to its "real" address (TODO: optimize?)
-          //TODO: currently broken, can't use XmiSerializationSharedData for
-          //this id mapping when merging, need local map
           try {
             int addr = getFsAddrForXmiId(id);
-            indexRep.addFS(addr);
+//            indexRep.addFS(addr);  // can't do now because sofa ref not yet fixed up
+              todo.add(addr);   // https://issues.apache.org/jira/browse/UIMA-4099
           } catch (NoSuchElementException e) {
             if (!lenient) {
               throw createException(XCASParsingException.UNKNOWN_ID, Integer.toString(id));
@@ -579,6 +602,28 @@ public class XmiCasDeserializer {
       }
     }
     
+    /**
+     * 
+     * @param sofaNum 1 if sofa null, or the sofa Xmi Id
+     * @return the FS Repository associated with the sofa xmiId
+     * @throws XCASParsingException
+     */
+    private FSIndexRepositoryImpl getIndexRepo(String sofa, int sofaXmiId) throws XCASParsingException {
+      // a view with no Sofa will be added to the 1st, _InitialView, index
+
+      if (sofa == null) {
+        return (FSIndexRepositoryImpl) indexRepositories.get(1);
+      }
+      // translate sofa's xmi:id into its sofanum
+      int sofaAddr;
+      try {
+        sofaAddr = getFsAddrForXmiId(sofaXmiId);
+      } catch (NoSuchElementException e) {
+        throw createException(XCASParsingException.UNKNOWN_ID, Integer.toString(sofaXmiId));
+      }
+      int sofaNum = casBeingFilled.getFeatureValue(sofaAddr, sofaNumFeatCode);
+      return (FSIndexRepositoryImpl) indexRepositories.get(sofaNum);
+    }
     
     /**
      * Handles the processing of a cas:View element in the XMI. The cas:View element encodes indexed
@@ -595,38 +640,45 @@ public class XmiCasDeserializer {
       // TODO: this requires View to come AFTER all of its members
       if (addmembersString != null) {
     	  processView(sofa, addmembersString);
-      } 
-      int sofaNum = 1;
-      FSIndexRepositoryImpl indexRep = null;
-      if (delmemberString != null || reindexmemberString != null) {
-    	if (sofa != null) {
-              // translate sofa's xmi:id into its sofanum
-              int sofaXmiId = Integer.parseInt(sofa);
-              int sofaAddr = getFsAddrForXmiId(sofaXmiId);
-              sofaNum = casBeingFilled.getFeatureValue(sofaAddr, sofaNumFeatCode);
-        }
-        indexRep = (FSIndexRepositoryImpl) indexRepositories.get(sofaNum);
+      }
+      if (delmemberString == null && reindexmemberString == null) {
+        return;
+      }
+      
+      final int sofaXmiId = (sofa == null) ? 1 : Integer.parseInt(sofa);
+      FSIndexRepositoryImpl indexRep = getIndexRepo(sofa, sofaXmiId);
 
-        // TODO: optimize by going straight to int[] without going through
-        // intermediate String[]?
-        if (delmemberString != null) {
-          String[] members = parseArray(delmemberString);
-          for (int i = 0; i < members.length; i++) {
-            int id = Integer.parseInt(members[i]);
-            if (!isNewFS(id)) {  //preexisting FS
-              if (this.allowPreexistingFS == AllowPreexistingFS.disallow) {
-          	    this.disallowedViewMemberEncountered = true;  //ignore but flag it. 
-          	    continue;
-              } else if (this.allowPreexistingFS == AllowPreexistingFS.ignore) {
-          	    continue;  //ignore
-              }
-          	}
+//      int sofaNum = 1;
+//      FSIndexRepositoryImpl indexRep = null;
+//      
+//    	if (sofa != null) {
+//              // translate sofa's xmi:id into its sofanum
+//              int sofaXmiId = Integer.parseInt(sofa);
+//              int sofaAddr = getFsAddrForXmiId(sofaXmiId);
+//              sofaNum = casBeingFilled.getFeatureValue(sofaAddr, sofaNumFeatCode);
+//        }
+//      indexRep = (FSIndexRepositoryImpl) indexRepositories.get(sofaNum);
+
+      // TODO: optimize by going straight to int[] without going through
+      // intermediate String[]?
+      if (delmemberString != null) {
+        PositiveIntSet localRemoves = toBeRemoved.getTodos(indexRep);
+        String[] members = parseArray(delmemberString);
+        for (int i = 0; i < members.length; i++) {
+          int id = Integer.parseInt(members[i]);
+          if (!isNewFS(id)) {  //preexisting FS
+            if (this.allowPreexistingFS == AllowPreexistingFS.disallow) {
+        	    this.disallowedViewMemberEncountered = true;  //ignore but flag it. 
+        	    continue;
+            } else if (this.allowPreexistingFS == AllowPreexistingFS.ignore) {
+        	    continue;  //ignore
+            }
+        	}
         	// have to map each ID to its "real" address (TODO: optimize?)
-        	//TODO: currently broken, can't use XmiSerializationSharedData for
-        	//this id mapping when merging, need local map
         	try {
-        	  int addr = getFsAddrForXmiId(id);
-        	  indexRep.removeFS(addr);
+        	  int addr = getFsAddrForXmiId(id);      	  
+  //      	  indexRep.removeFS(addr); // can't do now because sofa ref not yet fixed up
+        	  localRemoves.add(addr);     // https://issues.apache.org/jira/browse/UIMA-4099
         	} catch (NoSuchElementException e) {
         	  if (!lenient) {
                 throw createException(XCASParsingException.UNKNOWN_ID, Integer.toString(id));
@@ -635,56 +687,54 @@ public class XmiCasDeserializer {
         		this.sharedData.addOutOfTypeSystemViewMember(sofa, members[i]);
         	  }
         	}
-          }	
-        }
-        if (reindexmemberString != null) {
-          String[] members = parseArray(reindexmemberString);
-          for (int i = 0; i < members.length; i++) {
-            int id = Integer.parseInt(members[i]);
-            if (!isNewFS(id)) { //preexising FS
-                if (this.allowPreexistingFS == AllowPreexistingFS.disallow) {
-            	    this.disallowedViewMemberEncountered = true; //ignore but flag it.
-            	    continue;
-                } else if (this.allowPreexistingFS == AllowPreexistingFS.ignore) {
-            	    continue;  
-                }
-            }
-          	// have to map each ID to its "real" address (TODO: optimize?)
-          	//TODO: currently broken, can't use XmiSerializationSharedData for
-          	//this id mapping when merging, need local map
-          	try {
-          	  int addr = getFsAddrForXmiId(id);
-          	  indexRep.removeFS(addr);
-          	  indexRep.addFS(addr);
-          	} catch (NoSuchElementException e) {
-          	  if (!lenient) {
-                throw createException(XCASParsingException.UNKNOWN_ID, Integer.toString(id));
-          	  } else {
-          		//unknown view member may be an OutOfTypeSystem FS
-          		this.sharedData.addOutOfTypeSystemViewMember(sofa, members[i]);
-          	  }
-          	}
-          }	
-        }
+        }	
       }
+//      if (reindexmemberString != null) {
+//        String[] members = parseArray(reindexmemberString);
+//        for (int i = 0; i < members.length; i++) {
+//          int id = Integer.parseInt(members[i]);
+//          if (!isNewFS(id)) { //preexising FS
+//              if (this.allowPreexistingFS == AllowPreexistingFS.disallow) {
+//          	    this.disallowedViewMemberEncountered = true; //ignore but flag it.
+//          	    continue;
+//              } else if (this.allowPreexistingFS == AllowPreexistingFS.ignore) {
+//          	    continue;  
+//              }
+//          }
+//        	// have to map each ID to its "real" address (TODO: optimize?)
+//        	//TODO: currently broken, can't use XmiSerializationSharedData for
+//        	//this id mapping when merging, need local map
+//        	try {
+//        	  int addr = getFsAddrForXmiId(id);
+//        	  indexRep.removeFS(addr);
+//        	  indexRep.addFS(addr);
+//        	} catch (NoSuchElementException e) {
+//        	  if (!lenient) {
+//              throw createException(XCASParsingException.UNKNOWN_ID, Integer.toString(id));
+//        	  } else {
+//        		//unknown view member may be an OutOfTypeSystem FS
+//        		this.sharedData.addOutOfTypeSystemViewMember(sofa, members[i]);
+//        	  }
+//        	}
+//        }	
+//      }
     }
 
-    private void readFS(final int addr, Attributes attrs) throws SAXException {
+    private void readFS(final int fsAddr, Attributes attrs, final boolean isNewFs) throws SAXException {
       // Hang on to address for handle features encoded as child elements
-      this.currentAddr = addr;
+      this.currentAddr = fsAddr;
       int id = -1;
       String attrName, attrValue;
-      final int typeCode = casBeingFilled.getHeapValue(addr);
+      final int typeCode = casBeingFilled.getHeapValue(fsAddr);
       final Type type = casBeingFilled.getTypeSystemImpl().ll_getTypeForCode(typeCode);
       int thisSofaNum = 0;
       
-      //is it a new FS
-      try {
-        id = Integer.parseInt(attrs.getValue(ID_ATTR_NAME));
-      } catch (NumberFormatException e) {
-        throw createException(XCASParsingException.ILLEGAL_ID, attrs.getValue(ID_ATTR_NAME));
-      }
-      boolean newFS = this.isNewFS(id);
+//      //is it a new FS
+//      try {
+//        id = Integer.parseInt(attrs.getValue(ID_ATTR_NAME));
+//      } catch (NumberFormatException e) {
+//        throw createException(XCASParsingException.ILLEGAL_ID, attrs.getValue(ID_ATTR_NAME));
+//      }
       
       if (sofaTypeCode == typeCode) {
         String sofaID = attrs.getValue(CAS.FEATURE_BASE_NAME_SOFAID);
@@ -692,7 +742,7 @@ public class XmiCasDeserializer {
           // initial view Sofa always has sofaNum = 1
           thisSofaNum = 1;
         } else {
-          if (newFS) {	
+          if (isNewFs) {	
             thisSofaNum = this.nextSofaNum++;
           } else {
         	thisSofaNum = Integer.parseInt(attrs.getValue(CAS.FEATURE_BASE_NAME_SOFANUM));
@@ -701,14 +751,26 @@ public class XmiCasDeserializer {
       }
       
       this.featsSeen = null;
+      
+      // before looping over all features for this FS, remove this FS if in any index.
+      // we do this once, before the feature setting loop, because that loop may set a sofa Ref which is 
+      // invalid (to be fixed up later). But the removal code needs a valid sofa ref.
+      if (!isNewFs) {   
+        casBeingFilled.removeFromCorruptableIndexAnyViewSetCache(fsAddr, casBeingFilled.getAddbackSingle());
+        // else clause not needed because caller does ll_createFS which sets this anyways
+//      } else {  
+//        // need this to prevent using sofa ref before it's set
+//        casBeingFilled.setCacheNotInIndex(fsAddr);  // new FSs are not indexed (yet)
+      }
+      // loop over all features for this FS
       for (int i = 0; i < attrs.getLength(); i++) {
         attrName = attrs.getQName(i);
         attrValue = attrs.getValue(i);
         if (attrName.equals(ID_ATTR_NAME)) {
           try {
             id = Integer.parseInt(attrValue);
-            newFS = this.isNewFS(id);
-            if (sofaTypeCode != typeCode && !newFS) {
+//            newFS = this.isNewFS(id);  // we already specifically got this attribute
+            if (sofaTypeCode != typeCode && !isNewFs) {
               this.featsSeen = new IntVector(attrs.getLength());
             } else {
               this.featsSeen = null;
@@ -725,18 +787,22 @@ public class XmiCasDeserializer {
           } else if (sofaTypeCode == typeCode && attrName.equals(CAS.FEATURE_BASE_NAME_SOFANUM)) {
             attrValue = Integer.toString(thisSofaNum);
           }
-          int featCode = handleFeature(type, addr, attrName, attrValue, newFS);
+          int featCode = handleFeature(type, fsAddr, attrName, attrValue, isNewFs);
           //if processing delta cas preexisting FS, keep track of features that have
           //been deserialized.
-          if (this.featsSeen != null && !newFS && featCode != -1) {
+          if (this.featsSeen != null && !isNewFs && featCode != -1) {
             this.featsSeen.add(featCode); 
           }
         }
+      }  // end of all features loop
+      
+      if (!isNewFs) {
+        casBeingFilled.addbackSingle(fsAddr);
       }
       
-      if (sofaTypeCode == typeCode && newFS) {
+      if (sofaTypeCode == typeCode && isNewFs) {
         // If a Sofa, create CAS view to get new indexRepository
-        SofaFS sofa = (SofaFS) casBeingFilled.createFS(addr);
+        SofaFS sofa = (SofaFS) casBeingFilled.createFS(fsAddr);
         // also add to indexes so we can retrieve the Sofa later
         casBeingFilled.getBaseIndexRepository().addFS(sofa);
         CAS view = casBeingFilled.getView(sofa);
@@ -749,8 +815,8 @@ public class XmiCasDeserializer {
         ((CASImpl) view).registerView(sofa);
         views.add(view);
       }
-      deserializedFsAddrs.add(addr);
-      addFsAddrXmiIdMapping(addr, id);
+      deserializedFsAddrs.add(fsAddr);
+      addFsAddrXmiIdMapping(fsAddr, id);
     }
 
     // The definition of a null value. Any other value must be in the expected
@@ -759,14 +825,24 @@ public class XmiCasDeserializer {
       return ((val == null) || (val.length() == 0));
     }
 
-    private int handleFeature(final Type type, int addr, String featName, String featVal, boolean newFS) throws SAXException {
+    /**
+     * Deserialize one feature 
+     * @param type -
+     * @param fsAddr the address of the FS
+     * @param featName the feature name
+     * @param featVal the value of the feature
+     * @param isNewFS  true if this is a new FS
+     * @return feature code or -1 if no feature in this type system
+     * @throws SAXException if Type doesn't have the feature, and we're not in lenient mode
+     */
+    private int handleFeature(final Type type, int fsAddr, String featName, String featVal, final boolean isNewFS) throws SAXException {
       final FeatureImpl feat = (FeatureImpl) type.getFeatureByBaseName(featName);
       if (feat == null) {
         if (!this.lenient) {
           throw createException(XCASParsingException.UNKNOWN_FEATURE, featName);
         }
         else {
-          sharedData.addOutOfTypeSystemAttribute(addr, featName, featVal);
+          sharedData.addOutOfTypeSystemAttribute(fsAddr, featName, featVal);
         }
         return -1;
       }
@@ -775,19 +851,19 @@ public class XmiCasDeserializer {
       //only update Sofa data features and mime type feature. skip other features.
       //skip Sofa data features if Sofa data is already set.
       //these features may not be modified.
-      if (sofaTypeCode == casBeingFilled.getHeapValue(addr) && !isNewFS(addr) ) {
-    	if (featName.equals(CAS.FEATURE_BASE_NAME_SOFAID) || 
-    		  featName.equals(CAS.FEATURE_BASE_NAME_SOFANUM))   { 
-          return feat.getCode();
-    	} else if (featName.equals(CAS.FEATURE_BASE_NAME_SOFASTRING)  || 
-    			     featName.equals(CAS.FEATURE_BASE_NAME_SOFAURI) || 
-    			     featName.equals(CAS.FEATURE_BASE_NAME_SOFAARRAY)) {
-          int currVal = casBeingFilled.getFeatureValue(addr, feat.getCode());
-    	  if (currVal != 0) 
-    	    return feat.getCode();
-    	}
-      }	  
-      handleFeature(addr, feat.getCode(), featVal);
+      if (sofaTypeCode == casBeingFilled.getHeapValue(fsAddr) && !isNewFS) {
+      	if (featName.equals(CAS.FEATURE_BASE_NAME_SOFAID) || 
+      		  featName.equals(CAS.FEATURE_BASE_NAME_SOFANUM))   { 
+            return feat.getCode();
+      	} else if (featName.equals(CAS.FEATURE_BASE_NAME_SOFASTRING)  || 
+      			     featName.equals(CAS.FEATURE_BASE_NAME_SOFAURI) || 
+      			     featName.equals(CAS.FEATURE_BASE_NAME_SOFAARRAY)) {
+            int currVal = casBeingFilled.getFeatureValue(fsAddr, feat.getCode());
+      	  if (currVal != 0) 
+      	    return feat.getCode();
+      	}
+      }	 
+      handleFeature(fsAddr, feat.getCode(), featVal);
       return feat.getCode();
     }
 
@@ -823,6 +899,10 @@ public class XmiCasDeserializer {
           try {
             if (!emptyVal(featVal)) {
               if (featCode == sofaFeatCode) {
+                // Debugging in 12 2014 reveals that this code is never run.
+                // because the featuretype of the sofa feature of an annotation is "fs ref", not "int".
+                // These are "fixed up" later.
+                
                 // special handling for "sofa" feature of annotation. Need to change
                 // it from a sofa reference into a sofa number
                 int sofaXmiId = Integer.parseInt(featVal);
@@ -871,7 +951,7 @@ public class XmiCasDeserializer {
         }
         case LowLevelCAS.TYPE_CLASS_FS: {
           try {
-            if (!emptyVal(featVal)) { this.
+            if (!emptyVal(featVal)) { 
               casBeingFilled.setFeatureValue(addr, featCode, Integer.parseInt(featVal));
             }
           } catch (NumberFormatException e) {
@@ -922,10 +1002,10 @@ public class XmiCasDeserializer {
         }
           // For list types, we do the same as for array types UNLESS we're dealing with
           // the tail feature of another list node. In that case we do the usual FS deserialization.
-        case XmiCasSerializer.TYPE_CLASS_INTLIST:
-        case XmiCasSerializer.TYPE_CLASS_FLOATLIST:
-        case XmiCasSerializer.TYPE_CLASS_STRINGLIST:
-        case XmiCasSerializer.TYPE_CLASS_FSLIST: {
+        case CasSerializerSupport.TYPE_CLASS_INTLIST:
+        case CasSerializerSupport.TYPE_CLASS_FLOATLIST:
+        case CasSerializerSupport.TYPE_CLASS_STRINGLIST:
+        case CasSerializerSupport.TYPE_CLASS_FSLIST: {
           if (ts.ll_getFeatureForCode(featCode).isMultipleReferencesAllowed()) {
             // do the usual FS deserialization
             try {
@@ -1026,7 +1106,7 @@ public class XmiCasDeserializer {
           break;
         }
         */
-        case XmiCasSerializer.TYPE_CLASS_INTLIST: {
+        case CasSerializerSupport.TYPE_CLASS_INTLIST: {
           int listFS = casBeingFilled.getFeatureValue(addr, featCode);
           if (listFS == 0) {
             listFS = listUtils.createIntList(featVals);
@@ -1040,7 +1120,7 @@ public class XmiCasDeserializer {
           }
           break;
         }
-        case XmiCasSerializer.TYPE_CLASS_FLOATLIST: {
+        case CasSerializerSupport.TYPE_CLASS_FLOATLIST: {
           int listFS = casBeingFilled.getFeatureValue(addr, featCode);
           if (listFS == 0) {
             listFS = listUtils.createFloatList(featVals);
@@ -1054,7 +1134,7 @@ public class XmiCasDeserializer {
           }
           break;
         }
-        case XmiCasSerializer.TYPE_CLASS_STRINGLIST: {
+        case CasSerializerSupport.TYPE_CLASS_STRINGLIST: {
           int listFS = casBeingFilled.getFeatureValue(addr, featCode);
           if (listFS == 0) {
             listFS = listUtils.createStringList(featVals);
@@ -1068,7 +1148,7 @@ public class XmiCasDeserializer {
           }
           break;
         }
-        case XmiCasSerializer.TYPE_CLASS_FSLIST: {
+        case CasSerializerSupport.TYPE_CLASS_FSLIST: {
           // this call, in addition to creating the list in the CAS, also
           // adds each list node ID to the fsListNodesFromMultivaluedProperties list.
           // We need this so we can go back through later and reset the addresses of the
@@ -1117,23 +1197,23 @@ public class XmiCasDeserializer {
     private int createArray(int arrayType, List<String> values, int xmiId, int addr) {
       int casArray = -1; 
       if (addr > 0) { //non-shared preexisting
-	    if (values.size() == casBeingFilled.getLowLevelCAS().ll_getArraySize(addr)) {
-		  casArray = addr;
-		  updateExistingArray(arrayType, values, casArray);
-	    } else {
-		  casArray = createNewArray(arrayType, values);
-	    }
+  	    if (values.size() == casBeingFilled.getLowLevelCAS().ll_getArraySize(addr)) {
+  	      casArray = addr;
+  	      updateExistingArray(arrayType, values, casArray);
+  	    } else {
+  	      casArray = createNewArray(arrayType, values);
+  	    }
       }  else  if (xmiId == -1) { //non-shared new
     	  casArray = createNewArray(arrayType, values);
       }  else if (isNewFS(xmiId)) {  //shared new
-    	  casArray = createNewArray(arrayType,values);
+    	  casArray = createNewArray(arrayType, values);
       }  else { //shared preexisting
     	  casArray = getFsAddrForXmiId(xmiId);
     	  if (values.size() == casBeingFilled.getLowLevelCAS().ll_getArraySize(casArray)) {
       		updateExistingArray(arrayType, values, casArray);
-      	  } else {
+      	} else {
       		casArray = createNewArray(arrayType, values);
-      	  }
+      	}
       }
 
       deserializedFsAddrs.add(casArray);
@@ -1175,7 +1255,7 @@ public class XmiCasDeserializer {
       casArray = fs.getAddress();
       
       for (int i = 0; i < values.size(); i++) {
-        String stringVal = (String) values.get(i);
+        String stringVal = values.get(i);
         casBeingFilled.setArrayValueFromString(casArray, i, stringVal);
       }
       
@@ -1375,6 +1455,8 @@ public class XmiCasDeserializer {
         }
       }
     }
+    
+    
 
     /*
      * (non-Javadoc)
@@ -1382,21 +1464,55 @@ public class XmiCasDeserializer {
      * @see org.xml.sax.ContentHandler#endDocument()
      */
     public void endDocument() throws SAXException {
-      // Resolve ID references, and add FSs to indexes
+      // Resolve ID references
       for (int i = 0; i < deserializedFsAddrs.size(); i++) {
         finalizeFS(deserializedFsAddrs.get(i));
       }
+      
+      // add FSs to indexes
+      //   These come from the add list
+      // https://issues.apache.org/jira/browse/UIMA-4099
+      for (Entry<FSIndexRepositoryImpl, PositiveIntSet> e : toBeAdded.entrySet()) {
+        FSIndexRepositoryImpl indexRep = e.getKey();
+        final PositiveIntSet todo = e.getValue();
+        final IntListIterator it = todo.iterator();
+        while (it.hasNext()) {
+          indexRep.addFS(it.next());
+        }
+      }
+      
+      // remove FSs from indexes
+      for (Entry<FSIndexRepositoryImpl, PositiveIntSet> e : toBeRemoved.entrySet()) {
+        FSIndexRepositoryImpl indexRep = e.getKey();
+        final PositiveIntSet todo = e.getValue();
+        final IntListIterator it = todo.iterator();
+        while (it.hasNext()) {
+          indexRep.removeFS(it.next());
+        }
+      }
+
       for (int i = 0; i < fsListNodesFromMultivaluedProperties.size(); i++) {
         remapFSListHeads(fsListNodesFromMultivaluedProperties.get(i));
       }
       // time = System.currentTimeMillis() - time;
       // System.out.println("Done in " + new TimeSpan(time));
 
-      for (int i = 0; i < views.size(); i++) {
-        ((CASImpl) views.get(i)).updateDocumentAnnotation();
-      }
       
-      //check if disallowed fs was encoutered]
+      for (CAS view : views) {
+        AutoCloseable ac = view.protectIndexes();
+        try {
+          ((CASImpl) view).updateDocumentAnnotation();
+        } finally {
+          try {
+            ac.close();
+          } catch (Exception e1) {
+            assert(false);
+          }
+        }
+      }
+     
+      
+      //check if disallowed fs  encoutered]
       if (this.disallowedViewMemberEncountered) {
     	  CASRuntimeException e = new CASRuntimeException(
   		          CASRuntimeException.DELTA_CAS_PREEXISTING_FS_DISALLOWED,
@@ -1607,16 +1723,16 @@ public class XmiCasDeserializer {
     private final int classifyType(int type) {
       // For most most types
       if (listUtils.isIntListType(type)) {
-        return XmiCasSerializer.TYPE_CLASS_INTLIST;
+        return CasSerializerSupport.TYPE_CLASS_INTLIST;
       }
       if (listUtils.isFloatListType(type)) {
-        return XmiCasSerializer.TYPE_CLASS_FLOATLIST;
+        return CasSerializerSupport.TYPE_CLASS_FLOATLIST;
       }
       if (listUtils.isStringListType(type)) {
-        return XmiCasSerializer.TYPE_CLASS_STRINGLIST;
+        return CasSerializerSupport.TYPE_CLASS_STRINGLIST;
       }
       if (listUtils.isFsListType(type)) {
-        return XmiCasSerializer.TYPE_CLASS_FSLIST;
+        return CasSerializerSupport.TYPE_CLASS_FSLIST;
       }
       return casBeingFilled.ll_getTypeClass(type);
     }
@@ -1656,7 +1772,7 @@ public class XmiCasDeserializer {
         //if we're merging, then we use a local id map for FSs above the
         //merge point, since each of the different XMI CASes being merged
         //can use these same ids for different FSs.
-        Integer localAddr = (Integer)localXmiIdToFsAddrMap.get(xmiId);
+        Integer localAddr = localXmiIdToFsAddrMap.get(xmiId);
         if (localAddr != null) {
           return localAddr.intValue();
         } else {
@@ -1699,7 +1815,7 @@ public class XmiCasDeserializer {
       Iterator<String> iter = featVals.iterator();
       XmlElementName elemName = new XmlElementName(null,featName,featName);
       while (iter.hasNext()) {
-        ootsElem.childElements.add(new XmlElementNameAndContents(elemName, (String)iter.next()));
+        ootsElem.childElements.add(new XmlElementNameAndContents(elemName, iter.next()));
       }
     } 
     
@@ -1802,7 +1918,7 @@ public class XmiCasDeserializer {
    * @param sharedData
    *          data structure used to allow the XmiCasSerializer and XmiCasDeserializer to share
    *          information.
-   * @param mergePoint
+   * @param mergePoint (represented as an xmiId, not an fsAddr)
    *          used to support merging multiple XMI CASes. If the mergePoint is negative, "normal"
    *          deserialization will be done, meaning the target CAS will be reset and the entire XMI
    *          content will be deserialized. If the mergePoint is nonnegative (including 0), the
@@ -2030,7 +2146,7 @@ public class XmiCasDeserializer {
    */
   private String xmiElementName2uimaTypeName(String nsUri, String localName) throws SAXException {
     // check map first to see if we've already computed the namespace mapping
-    String uimaNamespace = (String) xmiNamespaceToUimaNamespaceMap.get(nsUri);
+    String uimaNamespace = xmiNamespaceToUimaNamespaceMap.get(nsUri);
     if (uimaNamespace == null) {
       // check for the special "no-namespace" URI, which is used for UIMA types with no namespace
       if (XmiCasSerializer.DEFAULT_NAMESPACE_URI.equals(nsUri)) {
