@@ -271,7 +271,7 @@ public class IntArrayRBTcommon {
    * For color (a boolean), the size for stopping doubling is 32 * 2 * 1024 * 1024,
    *   so the # of words is the same.
    * 
-   * @param requiredSize
+   * @param requiredSize -
    */
   
   protected void ensureCapacityKlrp(int requiredSize) {       
@@ -335,10 +335,6 @@ public class IntArrayRBTcommon {
     this.parent = ensureArrayCapacity(this.parent, requiredSize);
   }
   
-  protected void ensureCapacity(int requiredSize) {
-    this.color = ensureBooleanArraySize(this.color, requiredSize);
-  }
-
   // only called for krlp style
   private int[] maximize(int[] array) {
     if (array.length < MAXklrp0) {
@@ -350,7 +346,7 @@ public class IntArrayRBTcommon {
   }
   
   protected int newNode(final int k) {
-    // Make sure the tree is big enough to accomodate a new node.
+    // Make sure the tree is big enough to accommodate a new node.
 
     if (useklrp) {
       int lenKlrp = (klrp.length >> 2);
@@ -366,15 +362,15 @@ public class IntArrayRBTcommon {
       if (this.next >= lenKlrp) {
         ensureCapacityKlrp(this.next + 1);
       }
+    } else {
+      // not using klrp format
+      ensureCapacityNotKrlp(this.next + 1);
     }
     
     if (this.next >= this.color.length){
-      if (!useklrp) {
-        ensureCapacityNotKrlp(this.next + 1);
-      }
-      ensureCapacity(this.next + 1);
+      this.color = ensureBooleanArraySize(this.color, this.next + 1);        
     }
-
+    
     // assert(key.length > next);
     final int z = this.next;
     ++this.next;
@@ -451,7 +447,8 @@ public class IntArrayRBTcommon {
   }
   
   /**
-   * Find the first node such that k &lt;= key[node].
+   * @param k -
+   * @return the first node such that k &lt;= key[node].
    */
   protected int findKey(final int k) {
     return findKeyDown(k, this.root);
@@ -474,6 +471,8 @@ public class IntArrayRBTcommon {
 
   /**
    * Find the node such that key[node] &ge; k and key[previous(node)] &lt; k.
+   * @param k -
+   * @return the node such that key[node] &ge; k and key[previous(node)] &lt; k.
    */
   public int findInsertionPointNoDups(final int k) {
     int node = this.root;
@@ -786,7 +785,7 @@ public class IntArrayRBTcommon {
     if (this.color[node] == black) {
       buf.append(" BLACK");
     }
-    buf.append("\n");
+    buf.append('\n');
     printKeys(getLeft(node), offset + 2, buf);
     printKeys(getRight(node), offset + 2, buf);
   }
