@@ -388,46 +388,50 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
    * 
    * @see org.apache.uima.jcas.JCas#getType(int)
    */
-  public TOP_Type getType(int i) {
+  public TOP_Type getType(final int i) {
     if (i >= typeArray.length || null == typeArray[i]) {
-      // unknown ID. This may be due to a need to update the typeArray
-      // due to switching class loaders. This updating is done
-      // partially lazily - when needed, beyond the particular CAS instance and
-      // view that was being passed to a process method when the
-      // class loader switch was done.
-
-      // In order for this to work, all access to the typeArray must be
-      // via this getter.
-
-      // Try instantiating the entries in typeArray for this class loader
-      instantiateJCas_Types(this.sharedView.currentClassLoader);
-      if (i >= typeArray.length || null == typeArray[i]) {
-
-        // unknown ID. Attempt to get offending class.
-        Class cls = JCasRegistry.getClassForIndex(i);
-        if (cls != null) {
-          String typeName = cls.getName();
-          // is type in type system
-          if (this.casImpl.getTypeSystem().getType(typeName) == null) {
-            // no - report error that JCAS type was not defined in XML
-            // descriptor
-            CASRuntimeException casEx = new CASRuntimeException(
-                CASRuntimeException.JCAS_TYPE_NOT_IN_CAS, new String[] { typeName });
-            throw casEx;
-          } else {
-            // yes - there was some problem loading the _Type object
-            CASRuntimeException casEx = new CASRuntimeException(
-                CASRuntimeException.JCAS_MISSING_COVERCLASS, new String[] { typeName + "_Type" });
-            throw casEx;
-          }
-
-        } else {
-          throw new CASRuntimeException(CASRuntimeException.JCAS_UNKNOWN_TYPE_NOT_IN_CAS);
-        }
-      }
+      getTypeInit(i); 
     }
     return typeArray[i];
   }
+  
+  private void getTypeInit(final int i) {
+    // unknown ID. This may be due to a need to update the typeArray
+    // due to switching class loaders. This updating is done
+    // partially lazily - when needed, beyond the particular CAS instance and
+    // view that was being passed to a process method when the
+    // class loader switch was done.
+
+    // In order for this to work, all access to the typeArray must be
+    // via this getter.
+
+    // Try instantiating the entries in typeArray for this class loader
+    instantiateJCas_Types(this.sharedView.currentClassLoader);
+    if (i >= typeArray.length || null == typeArray[i]) {
+
+      // unknown ID. Attempt to get offending class.
+      Class cls = JCasRegistry.getClassForIndex(i);
+      if (cls != null) {
+        String typeName = cls.getName();
+        // is type in type system
+        if (this.casImpl.getTypeSystem().getType(typeName) == null) {
+          // no - report error that JCAS type was not defined in XML
+          // descriptor
+          CASRuntimeException casEx = new CASRuntimeException(
+              CASRuntimeException.JCAS_TYPE_NOT_IN_CAS, new String[] { typeName });
+          throw casEx;
+        } else {
+          // yes - there was some problem loading the _Type object
+          CASRuntimeException casEx = new CASRuntimeException(
+              CASRuntimeException.JCAS_MISSING_COVERCLASS, new String[] { typeName + "_Type" });
+          throw casEx;
+        }
+
+      } else {
+        throw new CASRuntimeException(CASRuntimeException.JCAS_UNKNOWN_TYPE_NOT_IN_CAS);
+      }
+    }
+  }
 
   /*
    * (non-Javadoc)
@@ -510,7 +514,6 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
    * 
    * @param cl
    *                class loader to switch to 
-   * @throws CASException passthru
    */
   public void switchClassLoader(ClassLoader cl) {
 
@@ -519,7 +522,7 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
       instantiateJCas_Types(cl);
     }
     final JCasSharedView sv = this.sharedView;
-    sv.cAddr2Jfs = (JCasHashMap) sv.cAddr2JfsByClassLoader.get(cl);
+    sv.cAddr2Jfs = sv.cAddr2JfsByClassLoader.get(cl);
     if (null == sv.cAddr2Jfs) {
       // initial size low because this is the use case where a Pear isolation is happening, 
       //   it is more likely that the PEAR is only doing a subset of the JCas things
@@ -676,7 +679,7 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
     Type topType = ts.getTopType();
     Type superType = null;
     while (typeIt.hasNext()) {
-      Type t = (Type) typeIt.next();
+      Type t = typeIt.next();
       if (builtInsWithNoJCas.contains(t.getName()))
         continue;
       // comment here
@@ -711,6 +714,7 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
    * @param cas
    *                a CAS instance
    * @return newly created and initialized JCas
+   * @throws CASException -
    */
   public static JCas getJCas(CASImpl cas) throws CASException {
     JCasImpl jcas = new JCasImpl(cas);
@@ -719,7 +723,7 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
       StringBuffer msg = new StringBuffer(100);
       for (Exception f : sv.errorSet) {
         msg.append(f.getMessage());
-        msg.append("\n");
+        msg.append('\n');
       }
       CASException e = new CASException(CASException.JCAS_INIT_ERROR,
           new String[] { msg.toString() });
@@ -786,6 +790,10 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
   // see comment under getType(int))
 
   static private class JCasFsGenerator implements FSGenerator {
+    // multiple reader threads in same CAS
+    static final ThreadLocal<Object[]> initArgsThreadLocal = new ThreadLocal<Object[]>() {
+      protected Object[] initialValue() { return new Object[2]; } };
+
     private final int type;
 
     private final Constructor c;
@@ -795,6 +803,7 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
     private final int sofaNbrFeatCode;
 
     private final int annotSofaFeatCode;
+    
 
     JCasFsGenerator(int type, Constructor c, boolean isSubtypeOfAnnotationBase,
         int sofaNbrFeatCode, int annotSofaFeatCode) {
@@ -810,8 +819,21 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
     // 2) a dereference of an existing FS
     // 3) an iterator
     public FeatureStructure createFS(int addr, CASImpl casView) {
-      Object[] initargs = new Object[2];
-      JCasImpl jcasView = null;
+      try {
+        JCasImpl jcasView = (JCasImpl) casView.getJCas();
+        TOP fs = jcasView.getJfsFromCaddr(addr);
+        if (null != fs) {
+          fs.jcasType = jcasView.getType(type);
+          return fs;
+        }       
+        return doCreateFS(addr, casView);
+      } catch (CASException e1) {
+        logAndThrow(e1, null);
+        return null;  // to avoid compile warning
+      }
+    }
+  
+    private FeatureStructure doCreateFS(int addr, CASImpl casView) {
       // this funny logic is because although the annotationView should always be set if
       // a type is a subtype of annotation, it isn't always set if an application uses low-level
       // api's. Rather than blow up, we limp along.
@@ -825,18 +847,11 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
       final CASImpl view = (null != maybeAnnotationView) ? maybeAnnotationView : casView;
 
       try {
-        jcasView = (JCasImpl) view.getJCas();
-      } catch (CASException e1) {
-        logAndThrow(e1, jcasView);
-      }
-
-      // Return eq fs instance if already created
-      TOP fs = jcasView.getJfsFromCaddr(addr);
-      if (null != fs) {
-        fs.jcasType = jcasView.getType(type);
-      } else {
+        JCasImpl jcasView = (JCasImpl) view.getJCas();
+        final Object[] initargs = initArgsThreadLocal.get();  
         initargs[0] = Integer.valueOf(addr);
         initargs[1] = jcasView.getType(type);
+        FeatureStructure fs = null;
         try {
           fs = (TOP) c.newInstance(initargs);
         } catch (IllegalArgumentException e) {
@@ -849,8 +864,11 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
           logAndThrow(e, jcasView);
         }
         jcasView.putJfsFromCaddr(addr, fs);
+        return fs;
+      } catch (CASException e1) {
+        logAndThrow(e1, null);
+        return null;
       }
-      return fs;
     }
 
     private void logAndThrow(Exception e, JCasImpl jcasView) {
@@ -863,10 +881,9 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
       throw casEx;
     }
 
-    private int getSofaNbr(int addr, CASImpl casView) {
-      final LowLevelCAS llCas = casView.getLowLevelCAS();
-      final int sofa = llCas.ll_getIntValue(addr, annotSofaFeatCode, false);
-      return (sofa == 0) ? 0 : llCas.ll_getIntValue(sofa, sofaNbrFeatCode);
+    private int getSofaNbr(final int addr, final CASImpl casView) {      
+      final int sofa = casView.ll_getIntValue(addr, annotSofaFeatCode, false);
+      return (sofa == 0) ? 0 : casView.ll_getIntValue(sofa, sofaNbrFeatCode);
     }
   }
 
@@ -1015,8 +1032,8 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
   /**
    * Internal - throw missing feature exception at runtime
    * 
-   * @param feat
-   * @param type
+   * @param feat -
+   * @param type -
    */
   public void throwFeatMissing(String feat, String type) {
     CASRuntimeException e = new CASRuntimeException(CASRuntimeException.INAPPROP_FEAT,
@@ -1054,9 +1071,9 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
 
   // * Implementation of part of the Cas interface as part of JCas*
 
-  /**
+  /*
    * (Internal Use only) called by the CAS reset function - clears the hashtable holding the
-   * associations.
+   * associations. 
    */
   public static void clearData(CAS cas) {
     JCasImpl jcas = (JCasImpl) ((CASImpl) cas).getExistingJCas();
@@ -1070,7 +1087,7 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
       sv.fsArray0L = null;
       sv.integerArray0L = null;
     }
-    sv.cAddr2Jfs = (JCasHashMap) sv.cAddr2JfsByClassLoader
+    sv.cAddr2Jfs = sv.cAddr2JfsByClassLoader
         .get(((CASImpl) cas).getJCasClassLoader());
   }
 
@@ -1561,4 +1578,22 @@ public class JCasImpl extends AbstractCas_ImplBase implements AbstractCas, JCas
     }
     return viewList.iterator();
   }
+
+  /* (non-Javadoc)
+   * @see org.apache.uima.jcas.JCas#protectIndexes()
+   */
+  @Override
+  public AutoCloseable protectIndexes() {
+    return casImpl.protectIndexes();
+  }
+
+  /* (non-Javadoc)
+   * @see org.apache.uima.jcas.JCas#protectIndexes(java.lang.Runnable)
+   */
+  @Override
+  public void protectIndexes(Runnable runnable) {
+    casImpl.protectIndexes(runnable);  
+  }
+  
+  
 }
