@@ -1418,11 +1418,11 @@ public class AnalysisEngine_implTest extends TestCase {
       // -- check SourceDocumentInformation FSs
       Iterator<AnnotationFS> sdiIter = outCas.getAnnotationIndex(sdiType).iterator();
       assertTrue(sdiIter.hasNext());
-      AnnotationFS outSdiFs = (AnnotationFS) sdiIter.next();
+      AnnotationFS outSdiFs = sdiIter.next();
       assertEquals("This is", outSdiFs.getCoveredText());
       assertEquals("cas1", outSdiFs.getStringValue(uriFeat));
       assertTrue(sdiIter.hasNext());
-      outSdiFs = (AnnotationFS) sdiIter.next();
+      outSdiFs = sdiIter.next();
       assertEquals(" one.", outSdiFs.getCoveredText());
       assertEquals("cas2", outSdiFs.getStringValue(uriFeat));
       assertFalse(sdiIter.hasNext());
@@ -1437,11 +1437,11 @@ public class AnalysisEngine_implTest extends TestCase {
       // -- check SourceDocumentInformation FSs
       sdiIter = outCas.getAnnotationIndex(sdiType).iterator();
       assertTrue(sdiIter.hasNext());
-      outSdiFs = (AnnotationFS) sdiIter.next();
+      outSdiFs = sdiIter.next();
       assertEquals("This is", outSdiFs.getCoveredText());
       assertEquals("cas2", outSdiFs.getStringValue(uriFeat));
       assertTrue(sdiIter.hasNext());
-      outSdiFs = (AnnotationFS) sdiIter.next();
+      outSdiFs = sdiIter.next();
       assertEquals(" two.", outSdiFs.getCoveredText());
       assertEquals("cas3", outSdiFs.getStringValue(uriFeat));
       assertFalse(sdiIter.hasNext());
@@ -1575,6 +1575,24 @@ public class AnalysisEngine_implTest extends TestCase {
     assertEquals("D/C/B/A/F/E/", TestAnnotator2.allContexts);
     cloneFile.delete();
   }
+
+  public void testMultiViewAnnotatorInput() throws Exception {
+    try {
+      AnalysisEngineDescription transAnnotatorDesc = UIMAFramework.getXMLParser().parseAnalysisEngineDescription(
+              new XMLInputSource(JUnitExtension
+                      .getFile("TextAnalysisEngineImplTest/MultiViewAnnotator.xml")));
+      PrimitiveAnalysisEngine_impl ae = new PrimitiveAnalysisEngine_impl();
+      ae.initialize(transAnnotatorDesc, null);
+      CAS tcas = ae.newCAS();
+      tcas.setDocumentText("this beer is good");
+      assertTrue(tcas.getView("_InitialView").getDocumentText().equals("this beer is good"));
+      ae.process(tcas);
+      assertTrue(tcas.getView("GermanDocument").getViewName().equals("GermanDocument"));
+      assertTrue(tcas.getView("GermanDocument").getDocumentText().equals("das bier ist gut"));
+    } catch (Exception e) {
+      JUnitExtension.handleException(e);
+    }
+  }
   
   /*
    * Get size of file asif has a single line-end character
