@@ -23,13 +23,14 @@ import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileInputStream;
+import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.PrintWriter;
 import java.io.StringReader;
 import java.io.StringWriter;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Stack;
 
 import javax.xml.parsers.FactoryConfigurationError;
 import javax.xml.parsers.ParserConfigurationException;
@@ -44,7 +45,6 @@ import org.apache.uima.cas.ArrayFS;
 import org.apache.uima.cas.CAS;
 import org.apache.uima.cas.CASRuntimeException;
 import org.apache.uima.cas.FSIndex;
-import org.apache.uima.cas.FSIndexRepository;
 import org.apache.uima.cas.FSIterator;
 import org.apache.uima.cas.Feature;
 import org.apache.uima.cas.FeatureStructure;
@@ -59,7 +59,6 @@ import org.apache.uima.cas.text.AnnotationFS;
 import org.apache.uima.cas_data.impl.CasComparer;
 import org.apache.uima.internal.util.XmlAttribute;
 import org.apache.uima.internal.util.XmlElementNameAndContents;
-import org.apache.uima.jcas.tcas.Annotation;
 import org.apache.uima.resource.metadata.FsIndexDescription;
 import org.apache.uima.resource.metadata.TypeDescription;
 import org.apache.uima.resource.metadata.TypeSystemDescription;
@@ -71,12 +70,10 @@ import org.apache.uima.util.CasCreationUtils;
 import org.apache.uima.util.FileUtils;
 import org.apache.uima.util.XMLInputSource;
 import org.apache.uima.util.XMLSerializer;
-import org.xml.sax.Attributes;
 import org.xml.sax.ContentHandler;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
 import org.xml.sax.XMLReader;
-import org.xml.sax.helpers.DefaultHandler;
 
 
 public class XmiCasDeserializerTest extends TestCase {
@@ -142,7 +139,8 @@ public class XmiCasDeserializerTest extends TestCase {
 
     // reserialize as XMI
     String xml = serialize(cas, null);
-//    System.out.println(xml);
+//    dumpStr2File(xml, "145");
+
     
     // deserialize into another CAS
     CAS cas2 = CasCreationUtils.createCas(typeSystemDescription, new TypePriorities_impl(), indexes);
@@ -335,7 +333,7 @@ public class XmiCasDeserializerTest extends TestCase {
     //serialize complete
     XmiSerializationSharedData sharedData = new XmiSerializationSharedData();
     String xml = serialize(cas1, sharedData);
-    System.out.println(xml);
+//    System.out.println(xml);
     int maxOutgoingXmiId = sharedData.getMaxXmiId();
 
     //deserialize into cas2
@@ -360,7 +358,7 @@ public class XmiCasDeserializerTest extends TestCase {
 
     // serialize cas2 in delta format
     String deltaxml1 = serialize(cas2, sharedData2, marker);
-    System.out.println(deltaxml1);
+//    System.out.println(deltaxml1);
 
     //deserialize delta xmi into cas1
     this.deserialize(deltaxml1, cas1, sharedData, true, maxOutgoingXmiId, AllowPreexistingFS.allow);
@@ -386,7 +384,7 @@ public class XmiCasDeserializerTest extends TestCase {
     //serialize complete
     XmiSerializationSharedData sharedData = new XmiSerializationSharedData();
     String xml = serialize(cas1, sharedData);
-    System.out.println(xml);
+//    System.out.println(xml);
     int maxOutgoingXmiId = sharedData.getMaxXmiId();
 
     //deserialize into cas2
@@ -416,7 +414,7 @@ public class XmiCasDeserializerTest extends TestCase {
     
     // serialize cas2 in delta format
     String deltaxml1 = serialize(cas2, sharedData2, marker);
-    System.out.println(deltaxml1);
+//    System.out.println(deltaxml1);
 
     //deserialize delta xmi into cas1
     this.deserialize(deltaxml1, cas1, sharedData, true, maxOutgoingXmiId, AllowPreexistingFS.allow);
@@ -440,7 +438,7 @@ public class XmiCasDeserializerTest extends TestCase {
     //serialize complete
     XmiSerializationSharedData sharedData = new XmiSerializationSharedData();
     String xml = serialize(cas1, sharedData);
-    System.out.println(xml);
+//    System.out.println(xml);
     int maxOutgoingXmiId = sharedData.getMaxXmiId();
 
     //deserialize into cas2
@@ -465,7 +463,7 @@ public class XmiCasDeserializerTest extends TestCase {
 
     // serialize cas2 in delta format
     String deltaxml1 = serialize(cas2, sharedData2, marker);
-    System.out.println(deltaxml1);
+//    System.out.println(deltaxml1);
 
     //deserialize delta xmi into cas1
     this.deserialize(deltaxml1, cas1, sharedData, true, maxOutgoingXmiId, AllowPreexistingFS.allow);
@@ -487,14 +485,19 @@ public class XmiCasDeserializerTest extends TestCase {
       CAS cas2 = cas.createView("OtherSofa");
       cas2.setDocumentText("This is only a test");
 
-      // create an annotation and add to index of both views
-      AnnotationFS anAnnot = cas.createAnnotation(cas.getAnnotationType(), 0, 5);
-      cas.getIndexRepository().addFS(anAnnot);
-      cas2.getIndexRepository().addFS(anAnnot);
-      FSIndex tIndex = cas.getAnnotationIndex();
-      FSIndex t2Index = cas2.getAnnotationIndex();
-      assertTrue(tIndex.size() == 2); // document annot and this one
-      assertTrue(t2Index.size() == 2); // ditto
+      // Change this test to create an instance of TOP because you cannot add an annotation to other than 
+      //   the view it is created in. https://issues.apache.org/jira/browse/UIMA-4099
+      // create a TOP and add to index of both views
+      Type topType = cas.getTypeSystem().getTopType();
+      FeatureStructure aTOP = cas.createFS(topType);
+      cas.getIndexRepository().addFS(aTOP);
+      cas2.getIndexRepository().addFS(aTOP); 
+      FSIterator<FeatureStructure> it = cas.getIndexRepository().getAllIndexedFS(topType);
+      FSIterator<FeatureStructure> it2 = cas2.getIndexRepository().getAllIndexedFS(topType);
+      it.next(); it.next();
+      it2.next(); it2.next(); 
+      assertFalse(it.hasNext());
+      assertFalse(it2.hasNext());
 
       // serialize
       StringWriter sw = new StringWriter();
@@ -521,8 +524,14 @@ public class XmiCasDeserializerTest extends TestCase {
         assertEquals("This is only a test", newCas2.getDocumentText());
 
         // check that annotation is still indexed in both views
-        assertTrue(tIndex.size() == 2); // document annot and this one
-        assertTrue(t2Index.size() == 2); // ditto
+        // check that annotation is still indexed in both views
+        it = newCas.getIndexRepository().getAllIndexedFS(topType);
+        it2 = newCas2.getIndexRepository().getAllIndexedFS(topType);
+        it.next(); it.next();
+        it2.next(); it2.next(); 
+        assertFalse(it.hasNext());
+//        assertFalse(it2.hasNext());        assertTrue(tIndex.size() == 2); // document annot and this one
+//        assertTrue(t2Index.size() == 2); // ditto
 
         newCas.reset();
       }
@@ -775,7 +784,7 @@ public class XmiCasDeserializerTest extends TestCase {
     XmiSerializationSharedData serSharedData = new XmiSerializationSharedData();
     String xmiStr = serialize(cas, serSharedData);
     int maxOutgoingXmiId = serSharedData.getMaxXmiId();
-    
+        
     // deserialize into two new CASes, each with its own instance of XmiSerializationSharedData 
     // so we can get consistent IDs later when serializing back.
     CAS newCas1 = CasCreationUtils.createCas(typeSystem, new TypePriorities_impl(), indexes);
@@ -804,7 +813,7 @@ public class XmiCasDeserializerTest extends TestCase {
     //to test links across merge boundary
     Type orgType = newCas2.getTypeSystem().getType(
             "org.apache.uima.testTypeSystem.Organization");
-    AnnotationFS org = (AnnotationFS)newCas2.getAnnotationIndex(orgType).iterator().next();
+    AnnotationFS org = newCas2.getAnnotationIndex(orgType).iterator().next();
     Type ownerType = newCas2.getTypeSystem().getType(
             "org.apache.uima.testTypeSystem.Owner");
     Feature argsFeat = ownerType.getFeatureByBaseName("relationArgs");
@@ -870,6 +879,7 @@ public class XmiCasDeserializerTest extends TestCase {
 
     // Serialize/deserialize again in case merge created duplicate ids
     String newSerCasMerged = serialize(cas, serSharedData);
+    
     deserialize(newSerCasMerged, cas, serSharedData, false, -1);
     
     //check covered text of annotations
@@ -900,11 +910,9 @@ public class XmiCasDeserializerTest extends TestCase {
     assertEquals(sofaText1, targetView1.getDocumentText());
     CAS targetView2 = cas.getView("newSofa2");
     assertEquals(sofaText2, targetView2.getDocumentText());
-    AnnotationFS targetAnnot1 = (AnnotationFS) 
-      targetView1.getAnnotationIndex(orgType).iterator().get();
+    AnnotationFS targetAnnot1 = targetView1.getAnnotationIndex(orgType).iterator().get();
     assertEquals(annotText, targetAnnot1.getCoveredText());
-    AnnotationFS targetAnnot2 = (AnnotationFS) 
-    targetView2.getAnnotationIndex(orgType).iterator().get();
+    AnnotationFS targetAnnot2 = targetView2.getAnnotationIndex(orgType).iterator().get();
     assertEquals(annotText, targetAnnot2.getCoveredText());
     assertTrue(targetView1.getSofa().getSofaRef() != 
             targetView2.getSofa().getSofaRef());
@@ -912,8 +920,7 @@ public class XmiCasDeserializerTest extends TestCase {
     CAS checkPreexistingView = cas.getView("preexistingView");
     assertEquals(preexistingViewText, checkPreexistingView.getDocumentText());
     Type personType = cas.getTypeSystem().getType("org.apache.uima.testTypeSystem.Person");    
-    AnnotationFS targetAnnot3 = (AnnotationFS)
-            checkPreexistingView.getAnnotationIndex(personType).iterator().get();
+    AnnotationFS targetAnnot3 = checkPreexistingView.getAnnotationIndex(personType).iterator().get();
     assertEquals("John Smith", targetAnnot3.getCoveredText());
 
     // Check the FS with an array of pre-existing FSs
@@ -989,7 +996,7 @@ public class XmiCasDeserializerTest extends TestCase {
 	  
 	  //check language feature of doc annot is not changed.
 	  //System.out.println(cas1.getDocumentAnnotation().getStringValue(languageF));
-	  assertTrue( ((FeatureStructure) cas1.getAnnotationIndex().iterator().next()).getStringValue(languageF).equals("x-unspecified"));
+	  assertTrue( cas1.getAnnotationIndex().iterator().next().getStringValue(languageF).equals("x-unspecified"));
 	  //check new annotation exists and preexisting is not deleted
 	  assertTrue(cas1.getAnnotationIndex().size()==4);
    } catch (Exception e) {
@@ -1049,7 +1056,7 @@ public class XmiCasDeserializerTest extends TestCase {
     	 
       //check language feature of doc annot is not changed.
       //System.out.println(cas1.getDocumentAnnotation().getStringValue(languageF));
-      assertTrue( ((FeatureStructure) cas1.getAnnotationIndex().iterator().next()).getStringValue(languageF).equals("x-unspecified"));
+      assertTrue( cas1.getAnnotationIndex().iterator().next().getStringValue(languageF).equals("x-unspecified"));
       //check new annotation exists
       assertTrue(cas1.getAnnotationIndex().size() == 3); // cas2 should be unchanged. 
 	} catch (Exception e) {
@@ -1373,7 +1380,7 @@ public class XmiCasDeserializerTest extends TestCase {
     //Test modification of a nonshared multivalued feature.
       //This should serialize the encompassing FS.
       Iterator<FeatureStructure> iter = cas2.getIndexRepository().getIndex("testEntityIndex").iterator();
-      FeatureStructure cas2EntityFS = (FeatureStructure) iter.next();
+      FeatureStructure cas2EntityFS = iter.next();
       StringArrayFS cas2strarrayFS = (StringArrayFS) cas2EntityFS.getFeatureValue(classesFeat);
       cas2strarrayFS.set(1, "class2");
       cas2strarrayFS.set(2, "class3");
@@ -1409,7 +1416,7 @@ public class XmiCasDeserializerTest extends TestCase {
       //serialize complete cas and deserialize into cas3 and compare with cas1.
       String fullxml = serialize(cas2, sharedData2);
       XmiSerializationSharedData sharedData3 = new XmiSerializationSharedData();
-      this.deserialize(fullxml, cas3, sharedData3, true,-1);
+      this.deserialize(fullxml, cas3, sharedData3, true, -1);
       CasComparer.assertEquals(cas1, cas3); 
       
       //System.out.println("CAS1 " + serialize(cas1, new XmiSerializationSharedData()));
@@ -1579,14 +1586,13 @@ public class XmiCasDeserializerTest extends TestCase {
     assertEquals(1,sharedData2.getOutOfTypeSystemElements().size());
     OotsElementData ootsFeats3 = sharedData2.getOutOfTypeSystemFeatures(sharedData2.getFsAddrForXmiId(3));
     assertEquals(1, ootsFeats3.attributes.size());
-    XmlAttribute ootsAttr = (XmlAttribute)ootsFeats3.attributes.get(0);
+    XmlAttribute ootsAttr = ootsFeats3.attributes.get(0);
     assertEquals("mentionType", ootsAttr.name);
     assertEquals("NAME", ootsAttr.value);
     OotsElementData ootsFeats5 = sharedData2.getOutOfTypeSystemFeatures(sharedData2.getFsAddrForXmiId(5));
     assertEquals(0, ootsFeats5.attributes.size());
     assertEquals(1, ootsFeats5.childElements.size());
-    XmlElementNameAndContents ootsChildElem = (XmlElementNameAndContents)
-            ootsFeats5.childElements.get(0);
+    XmlElementNameAndContents ootsChildElem = ootsFeats5.childElements.get(0);
     assertEquals("mentionType", ootsChildElem.name.qName);
     assertEquals("NAME", ootsChildElem.contents);
     
@@ -1714,7 +1720,7 @@ public class XmiCasDeserializerTest extends TestCase {
     List ootsElems = sharedData.getOutOfTypeSystemElements();
     assertEquals(2, ootsElems.size());
     OotsElementData oed = sharedData.getOutOfTypeSystemFeatures(listFs.hashCode());
-    XmlAttribute attr = (XmlAttribute)oed.attributes.get(0);
+    XmlAttribute attr = oed.attributes.get(0);
     assertNotNull(attr);
     assertEquals(CAS.FEATURE_BASE_NAME_HEAD, attr.name);
     assertEquals(attr.value, ((OotsElementData)ootsElems.get(0)).xmiId);
@@ -1771,21 +1777,37 @@ public class XmiCasDeserializerTest extends TestCase {
     CasComparer.assertEquals(originalCas, newCas);    
  }
   
-  public void testGetNumChildren() throws Exception {
-    // deserialize a complex XCAS
-    CAS cas = CasCreationUtils.createCas(typeSystem, null, indexes);
-//    InputStream serCasStream = new FileInputStream(JUnitExtension.getFile("ExampleCas/cas.xml"));
-//    XCASDeserializer.deserialize(serCasStream, cas);
+//  public void testdebugnc() throws Exception {
+//    CAS cas = CasCreationUtils.createCas(typeSystem, null, indexes);
+//    InputStream serCasStream = new FileInputStream(JUnitExtension.getFile("ExampleCas/simpleCas.xmi"));
+//    XmiCasDeserializer.deserialize(serCasStream, cas);
 //    serCasStream.close();
-  InputStream serCasStream = new FileInputStream(JUnitExtension.getFile("ExampleCas/simpleCas.xmi"));
-  XmiCasDeserializer.deserialize(serCasStream, cas);
-  serCasStream.close();
-    
-    // call serializer with a ContentHandler that checks numChildren
-    XmiCasSerializer xmiSer = new XmiCasSerializer(cas.getTypeSystem());
-    GetNumChildrenTestHandler handler = new GetNumChildrenTestHandler(xmiSer);
-    xmiSer.serialize(cas, handler);
-  }
+//    
+//    XmiSerializationSharedData sharedData = new XmiSerializationSharedData();
+//    String r = serialize(cas, sharedData);
+//    
+//    {  File f = new File("C:/au/wksp431/apache/json-work/tempdebug.xml");
+//  PrintWriter pw = new PrintWriter(f);
+//  pw.println(r);
+//  pw.close();
+//    }
+//
+//  }
+//  public void testGetNumChildren() throws Exception {
+//    // deserialize a complex XCAS
+//    CAS cas = CasCreationUtils.createCas(typeSystem, null, indexes);
+////    InputStream serCasStream = new FileInputStream(JUnitExtension.getFile("ExampleCas/cas.xml"));
+////    XCASDeserializer.deserialize(serCasStream, cas);
+////    serCasStream.close();
+//  InputStream serCasStream = new FileInputStream(JUnitExtension.getFile("ExampleCas/simpleCas.xmi"));
+//  XmiCasDeserializer.deserialize(serCasStream, cas);
+//  serCasStream.close();
+//    
+//    // call serializer with a ContentHandler that checks numChildren
+//    XmiCasSerializer xmiSer = new XmiCasSerializer(cas.getTypeSystem());
+//    GetNumChildrenTestHandler handler = new GetNumChildrenTestHandler(xmiSer, (CASImpl)cas);
+//    xmiSer.serialize(cas, handler, handler.tcds);
+//  }
 
   /** Utility method for serializing a CAS to an XMI String 
    * */
@@ -1860,52 +1882,81 @@ public class XmiCasDeserializerTest extends TestCase {
     }
     return true;
   }  
-  
-  static class GetNumChildrenTestHandler extends DefaultHandler {
-    XmiCasSerializer xmiSer;
-    Stack<Integer> childCountStack = new Stack<Integer>();
-    
-    GetNumChildrenTestHandler(XmiCasSerializer xmiSer) {
-      this.xmiSer = xmiSer;
-      childCountStack.push(Integer.valueOf(1));
-    }
 
-    /* (non-Javadoc)
-     * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
-     */
-    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
-      // TODO Auto-generated method stub
-      super.startElement(uri, localName, qName, attributes);
-      childCountStack.push(Integer.valueOf(xmiSer.getNumChildren()));
-    }
+  /** for debug
+   * 
+   * @param s - string to dump
+   * @param name - file name part
+   */
+  private static void dumpStr2File(String s, String namepart) throws FileNotFoundException {
+ 
+    File f = new File("C:/au/wksp431/apache/json-work/temp" + namepart + ".xml");
+    PrintWriter pw = new PrintWriter(f);
+    pw.println(s);
+    pw.close();
 
-    /* (non-Javadoc)
-     * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
-     */
-    public void endElement(String uri, String localName, String qName) throws SAXException {
-      // TODO Auto-generated method stub
-      super.endElement(uri, localName, qName);
-      //check that we've seen the expected number of child elements
-      //(count on top of stack should be 0)
-      Integer count = (Integer)childCountStack.pop();
-      assertEquals(0, count.intValue());
-      
-      //decremenet child count of our parent
-      count = (Integer)childCountStack.pop();
-      childCountStack.push(Integer.valueOf(count.intValue() - 1));
-    }
-
-    /* (non-Javadoc)
-     * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
-     */
-    public void characters(char[] ch, int start, int length) throws SAXException {
-      // text node is considered a child
-      if (length > 0) {
-        Integer count = (Integer)childCountStack.pop();
-        childCountStack.push(Integer.valueOf(count.intValue() - 1));
-      }
-    }
-
-    
   }
+//  /**
+//   * Extends the SAX Normal default handler used to output xml
+//   * Keeps a stack of counts:
+//   *    For every startElement call, pushes the number of children of that element onto the stack
+//   *    For every endElement call, test if at number has been decremented to 0
+//   *       Inner endElement calls decrement the count (of their parentt,
+//   *       Inner "text" elements also do this (for their parent)
+//   */
+//  static class GetNumChildrenTestHandler extends DefaultHandler {
+//    XmiCasSerializer xmiSer;
+//    Stack<Integer> childCountStack = new Stack<Integer>();
+//    private XmiDocSerializer tcds; 
+//    
+//    GetNumChildrenTestHandler(XmiCasSerializer xmiSer, CASImpl cas) {
+//      this.xmiSer = xmiSer;
+//      tcds = xmiSer.getTestXmiDocSerializer(this, cas);
+//      childCountStack.push(Integer.valueOf(1));
+//    }
+//
+//    /* (non-Javadoc)
+//     * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
+//     */
+//    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
+//      // TODO Auto-generated method stub
+//      super.startElement(uri, localName, qName, attributes);
+//      childCountStack.push(Integer.valueOf(tcds.getNumChildren()));
+//      System.out.format("Debug: NumberOfChildren: Starting element %s,  setting count to %d%n", qName, tcds.getNumChildren());
+//    }
+//
+//    /* (non-Javadoc)
+//     * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
+//     */
+//    public void endElement(String uri, String localName, String qName) throws SAXException {
+//      // TODO Auto-generated method stub
+//      super.endElement(uri, localName, qName);
+//      //check that we've seen the expected number of child elements
+//      //(count on top of stack should be 0)
+//      Integer count = (Integer)childCountStack.pop();
+//      assertEquals(0, count.intValue());
+//      
+//      //decremenet child count of our parent
+//      count = (Integer)childCountStack.pop();
+//      childCountStack.push(Integer.valueOf(count.intValue() - 1));
+//      System.out.format("Debug: NumberOfChildren: ending element %s,  decr parent count to %d%n", qName, count - 1);
+//
+//    }
+//
+//    /* (non-Javadoc)
+//     * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
+//     */
+//    public void characters(char[] ch, int start, int length) throws SAXException {
+//      // text node is considered a child
+//      if (length > 0) {
+//        Integer count = (Integer)childCountStack.pop();
+//        childCountStack.push(Integer.valueOf(count.intValue() - 1));
+//        System.out.format("Debug: NumberOfChildren: ending text element,  decr parent count to %d%n", count - 1);        
+//      } else {
+//        System.out.format("Debug: NumberOfChildren: ending empty text element, not decr count%n");
+//      }
+//    }
+//
+//    
+//  }
 }
