@@ -25,6 +25,7 @@ import java.util.ArrayList;
 import java.util.ConcurrentModificationException;
 import java.util.Iterator;
 import java.util.List;
+import java.util.NoSuchElementException;
 
 import junit.framework.TestCase;
 
@@ -87,6 +88,33 @@ public class IteratorTest extends TestCase {
   private Type sentenceType;
   
   private Type subsentenceType;
+  
+  private FSIndex<FeatureStructure> bagIndex;
+  private FSIndex<FeatureStructure> ssBagIndex;
+  
+  private FSIndex<FeatureStructure> setIndex;
+  private FSIndex<FeatureStructure> ssSetIndex;
+
+  private FSIndex<FeatureStructure> sortedIndex;
+  private FSIndex<FeatureStructure> ssSortedIndex;
+  
+  private FSIndex<?> jcasBagIndex;
+  private FSIndex<?> jcasSsBagIndex;
+  
+  private FSIndex<?> jcasSetIndex;
+  private FSIndex<?> jcasSsSetIndex;
+
+  private FSIndex<?> jcasSortedIndex;
+  private FSIndex<?> jcasSsSortedIndex;
+  
+  private JCas jcas;
+
+  private FSIndex<FeatureStructure> wordSetIndex;
+  private FSIndex<FeatureStructure> ssWordSetIndex;
+
+  private Feature wordFeat;
+
+  private Type wType;
 
   /**
    * Constructor for FilteredIteratorTest.
@@ -97,7 +125,7 @@ public class IteratorTest extends TestCase {
     super(arg0);
   }
 
-  public void setUp() {
+  public void setUp() throws CASException {
     // try {
     // this.cas = (CASImpl) CASInitializer.initCas(new CASTestSetup());
     // assertTrue(this.cas != null);
@@ -117,6 +145,7 @@ public class IteratorTest extends TestCase {
       ResourceSpecifier spec = (ResourceSpecifier) parser.parse(new XMLInputSource(descriptorFile));
       AnalysisEngine ae = UIMAFramework.produceAnalysisEngine(spec);
       this.cas = (CASImpl) ae.newCAS();
+      this.jcas = cas.getJCas();
       assertTrue(this.cas != null);
       this.ts = this.cas.getTypeSystem();
       assertTrue(this.ts != null);
@@ -176,6 +205,48 @@ public class IteratorTest extends TestCase {
     this.annotationType = null;
   }
   
+  private void setupindexes () {
+    bagIndex = this.cas.getIndexRepository().getIndex(CASTestSetup.ANNOT_BAG_INDEX);
+    setIndex = this.cas.getIndexRepository().getIndex(CASTestSetup.ANNOT_SET_INDEX);
+    sortedIndex = this.cas.getIndexRepository().getIndex(CASTestSetup.ANNOT_SORT_INDEX);
+    
+    ssBagIndex = bagIndex.withSnapshotIterators();
+    ssSetIndex = setIndex.withSnapshotIterators();
+    ssSortedIndex = sortedIndex.withSnapshotIterators();
+    
+    jcasBagIndex = this.jcas.getIndexRepository().getIndex(CASTestSetup.ANNOT_BAG_INDEX);
+    jcasSetIndex = this.jcas.getIndexRepository().getIndex(CASTestSetup.ANNOT_SET_INDEX);
+    jcasSortedIndex = this.jcas.getIndexRepository().getIndex(CASTestSetup.ANNOT_SORT_INDEX);
+    
+    jcasSsBagIndex = jcasBagIndex.withSnapshotIterators();
+    jcasSsSetIndex = jcasSetIndex.withSnapshotIterators();
+    jcasSsSortedIndex = jcasSortedIndex.withSnapshotIterators();
+
+  }
+  
+  public void testEmptySnapshotIterator() {
+    setupindexes();
+    FSIterator<FeatureStructure> it = sortedIndex.iterator();
+    assertFalse(it.isValid());
+    boolean ok = false;
+    try {
+      it.get();
+    } catch (NoSuchElementException e) {
+      ok = true;
+    }
+    assertTrue(ok);
+    
+    it = ssSortedIndex.iterator();
+    assertFalse(it.isValid());
+    ok = false;
+    try {
+      it.get();
+    } catch (NoSuchElementException e) {
+      ok = true;
+    }
+    assertTrue(ok);    
+  }
+  
   public void testGetIndexes() {
     Iterator<FSIndex<FeatureStructure>> it = this.cas.getIndexRepository().getIndexes();
     while (it.hasNext()) {
@@ -197,13 +268,85 @@ public class IteratorTest extends TestCase {
     }
     AnnotationFS match = this.cas.createAnnotation(this.annotationType, start, end);
     FSIndex<AnnotationFS> index = this.cas.getAnnotationIndex();
+    FSIndex<AnnotationFS> ssIndex = index.withSnapshotIterators();
     FSIterator<AnnotationFS> it = index.iterator();
+    FSIterator<AnnotationFS> ssit = ssIndex.iterator();
     it.moveTo(match);
+    ssit.moveTo(match);
     assertTrue(index.compare(match, it.get()) == 0);
+    assertTrue(index.compare(match, ssit.get()) == 0);
+    
     // The contract of moveTo() says that any preceding FS must be smaller.
     it.moveToPrevious();
+    ssit.moveToPrevious();
     assertTrue(index.compare(match, it.get()) > 0);
+    assertTrue(index.compare(match, ssit.get()) > 0);
   }
+  
+  public void testMoveToPastEnd() {  // https://issues.apache.org/jira/browse/UIMA-4094
+    this.cas.getIndexRepository().addFS(
+        this.cas.createAnnotation(this.annotationType, 1,2));
+    
+    AnnotationFS pastEnd = this.cas.createAnnotation(this.annotationType,  2,  3);
+    FSIndex<AnnotationFS> index = this.cas.getAnnotationIndex();
+    FSIterator<AnnotationFS> it = index.iterator();
+    it.moveTo(pastEnd);
+    assertFalse(it.isValid());
+    
+    index = index.withSnapshotIterators();
+    it = index.iterator();
+    it.moveTo(pastEnd);
+    assertFalse(it.isValid());
+
+  }
+  
+  public void testMoveToFirstOfEqualOneType() {
+    for (int i = 0; i < 2; i++) {
+      cas.reset();
+      if (i == 0) {
+        this.cas.getIndexRepository().addFS(
+            this.cas.createAnnotation(this.subsentenceType, 0, 1));        
+      }
+      this.cas.getIndexRepository().addFS(
+          this.cas.createAnnotation(this.subsentenceType, 1,2));
+      this.cas.getIndexRepository().addFS(
+          this.cas.createAnnotation(this.subsentenceType, 1,2));
+      this.cas.getIndexRepository().addFS(
+          this.cas.createAnnotation(this.subsentenceType, 1,3));
+      this.cas.getIndexRepository().addFS(
+          this.cas.createAnnotation(this.subsentenceType, 2,2));
+      this.cas.getIndexRepository().addFS(
+          this.cas.createAnnotation(this.subsentenceType, 2, 5));
+      
+      AnnotationFS testAnnot = this.cas.createAnnotation(this.subsentenceType, 1, 2);
+      
+      FSIndex<AnnotationFS> index = this.cas.getAnnotationIndex(this.subsentenceType);
+      FSIterator<AnnotationFS> it = index.iterator();
+      it.moveTo(testAnnot);
+      for (int j = 0; j < 2; j++) {
+        assertTrue(it.isValid());
+        AnnotationFS fs = it.get();
+        assertEquals(1, fs.getBegin()); 
+        assertEquals(2, fs.getEnd());
+        it.moveToNext();
+      }
+      assertTrue(it.isValid());
+      
+      index = index.withSnapshotIterators();
+      it = index.iterator();
+      it.moveTo(testAnnot);
+      for (int j = 0; j < 2; j++) {
+        assertTrue(it.isValid());
+        AnnotationFS fs = it.get();
+        assertEquals(1, fs.getBegin()); 
+        assertEquals(2, fs.getEnd());
+        it.moveToNext();
+      }
+      assertTrue(it.isValid());
+
+    }
+  }
+  
 
   private void createFSs(int i) {
     this.cas.getIndexRepository().addFS(
@@ -240,8 +383,10 @@ public class IteratorTest extends TestCase {
     
     System.out.println("test multicore iterator with " + numberOfCores + " threads");
     Thread[] threads = new Thread[numberOfCores];
-    final Throwable[] tthrowable = new Throwable[1];
+    
+    final Throwable[] tthrowable = new Throwable[1];  // trick to get a return value in a parameter
     tthrowable[0] = null;
+    
     for (int r = 0; r < 10; r++) {
       for (int i = 0; i < numberOfCores; i++) {
         final int finalI = i;
@@ -280,200 +425,55 @@ public class IteratorTest extends TestCase {
   public void testIterator() {
     setupFSs();
     
-    FSIndex<FeatureStructure> bagIndex = this.cas.getIndexRepository().getIndex(
-        CASTestSetup.ANNOT_BAG_INDEX);
-    FSIndex<FeatureStructure> setIndex = this.cas.getIndexRepository().getIndex(
-        CASTestSetup.ANNOT_SET_INDEX);
-    FSIndex<FeatureStructure> sortedIndex = this.cas.getIndexRepository().getIndex(
-        CASTestSetup.ANNOT_SORT_INDEX);
+    setupindexes();
     
     setIteratorWithoutMods(setIndex, -1);
-
-    // test find()
-    AnnotationFS annot = (AnnotationFS) setIndex.iterator().get();
-    assertNotNull(setIndex.find(annot));
-    assertNull(setIndex.find(this.cas.createAnnotation(this.annotationType, -1, -1)));
-
-    // do same for JCas
-    JCas jcas = null;
-    try {
-      jcas = this.cas.getJCas();
-    } catch (CASException e1) {
-      e1.printStackTrace();
-      assertTrue(false);
-    }
-    FSIndex<?> jcasSetIndex = jcas.getJFSIndexRepository().getIndex(CASTestSetup.ANNOT_SET_INDEX);
-    Annotation jcasAnnotation = (Annotation) jcasSetIndex.find(annot);
-    assertNotNull(jcasAnnotation);
-    assertNull(jcasSetIndex.find(this.cas.createAnnotation(this.annotationType, -1, -1)));
-
-    // /////////////////////////////////////////////////////////////////////////
-    // Test fast fail.
-
-    FSIterator<FeatureStructure> it = bagIndex.iterator(); // use bag index, remove add last one
-    AnnotationFS a = null;
-    // (preserves order for other tests).
-    it.moveToLast();
-    a = (AnnotationFS) it.get();
-    it = setIndex.iterator(); // back to set iterator to do testing
-    this.cas.getIndexRepository().removeFS(a);
-    this.cas.getIndexRepository().addFS(a);
-
-    boolean ok = false;
-    try {
-      it.next(); // should throw
-    } catch (ConcurrentModificationException e) {
-      ok = true;
-    }
-    assertTrue(ok);
-    ok = false;
-    try {
-      it.next(); // should throw
-    } catch (ConcurrentModificationException e) {
-      ok = true;
-    }
-    assertTrue(ok);
-    it.moveTo(a);
-    ok = false;
-    try {
-      it.next(); // should not throw
-      ok = true;
-    } catch (ConcurrentModificationException e) {
-      // checking this with the ok variable
-    }
-    assertTrue(ok);
-
-    // Test find()
-    Type wType = this.cas.getTypeSystem().getType("org.apache.uima.cas.test.types.Word");
-
-    Feature wordFeat = wType.getFeatureByBaseName("word");
-
-    for (int i = 0; i < 20; i++) {
-      FeatureStructure fs = this.cas.createFS(wType);
-      fs.setStringValue(wordFeat, "word" + i);
-      this.cas.getIndexRepository().addFS(fs);
-    }
-
-    FSIndex<FeatureStructure> wordSetIndex = this.cas.getIndexRepository().getIndex(
-        "Word Set Index");
-
-    it = wordSetIndex.iterator();
-
-    FeatureStructure fs = this.cas.createFS(wType);
-    fs.setStringValue(wordFeat, "word1");
-
-    // TEST moveTo() and get()
-    it.moveTo(fs);
-
-    assertSame(fs.getType(), it.get().getType());
-
-    Type t1 = fs.getType();
-    Type t2 = wordSetIndex.find(fs).getType();
-    assertSame(t1, t2);
+    setIteratorWithoutMods(ssSetIndex, -2);
     
     sortedIteratorWithoutMods(sortedIndex);
-
-    // /////////////////////////////////////////////////////////////////////////
-    // Test fast fail.
-
-    it = bagIndex.iterator(); // use bag index, remove add last one
-    // (preserves order for other tests).
-    it.moveToLast();
-    a = (AnnotationFS) it.get();
-    // for (it = sortedIndex.iterator(); it.hasNext(); it.next()) {
-    // System.out.println(it.get());
-    // }
-    it = sortedIndex.iterator();
-    it.next();
-    it.next();
-    this.cas.getIndexRepository().removeFS(a);
-    this.cas.getIndexRepository().addFS(a);
-    ok = false;
-    try {
-      it.get(); // should throw
-    } catch (ConcurrentModificationException e) {
-      ok = true;
-    }
-    assertTrue(ok);
-    ok = false;
-    try {
-      it.next(); // should throw
-    } catch (ConcurrentModificationException e) {
-      ok = true;
-    }
-    assertTrue(ok);
-    ok = false;
-    try {
-      it.moveToNext(); // should throw
-    } catch (ConcurrentModificationException e) {
-      ok = true;
-    }
-    assertTrue(ok);
-    it.moveTo(a);
-    ok = false;
-    try {
-      it.next(); // should not throw
-      ok = true;
-    } catch (ConcurrentModificationException e) {
-      // checking with boolean "ok"
-    }
-    assertTrue(ok);
-
-    sortedIndex = null;
-
+    sortedIteratorWithoutMods(ssSortedIndex);
+    
     bagIteratorWithoutMods(bagIndex);
+    bagIteratorWithoutMods(ssBagIndex);
+
+    // test find()
+    findTst(setIndex, jcasSetIndex);
+    findTst(ssSetIndex, jcasSsSetIndex);
+    findTst(sortedIndex, jcasSortedIndex);
+    findTst(ssSortedIndex, jcasSsSortedIndex);
+    findTst(bagIndex, jcasBagIndex);
+    findTst(ssBagIndex, jcasSsBagIndex);
+    
+    basicRemoveAdd(bagIndex, 20, 21);
+    basicRemoveAdd(ssBagIndex, 20, 21);
+    basicRemoveAdd(sortedIndex, 38, 39);
+    basicRemoveAdd(ssSortedIndex, 38, 39);
+    basicRemoveAdd(setIndex, 38, 39);
+    basicRemoveAdd(ssSetIndex, 38, 39);
+
     // /////////////////////////////////////////////////////////////////////////
     // Test fast fail.
 
-    it = bagIndex.iterator(); // use bag index, remove add last one
-    // (preserves order for other tests).
-    it.moveToLast();
-    a = (AnnotationFS) it.get();
-    this.cas.getIndexRepository().removeFS(a);
-    this.cas.getIndexRepository().addFS(a);
-
-    ok = false;
-    try {
-      it.get(); // should throw
-    } catch (ConcurrentModificationException e) {
-      ok = true;
-    }
-    assertTrue(ok);
-    it.moveToLast();
-    it.moveToPrevious();
-    it.moveToPrevious();
-    it.moveToPrevious();
-    it.moveToNext();
-    this.cas.getIndexRepository().removeFS(a);
-    this.cas.getIndexRepository().addFS(a);
-
-    ok = false;
-    try {
-      it.moveToNext(); // should throw
-    } catch (ConcurrentModificationException e) {
-      ok = true;
-    }
-    assertTrue(ok);
-    ok = false;
-    try {
-      it.moveToPrevious(); // should throw
-    } catch (ConcurrentModificationException e) {
-      ok = true;
-    }
-    assertTrue(ok);
-    it.moveTo(a);
-    ok = false;
-    try {
-      it.next(); // should not throw
-      ok = true;
-    } catch (ConcurrentModificationException e) {
-      // checking with boolean "ok"
-    }
-    assertTrue(ok);
+    fastFailTst(setIndex, true);
+    fastFailTst(ssSetIndex, false);
+    
+    fastFailTst(bagIndex, true);
+    fastFailTst(ssBagIndex, false);
+   
+    fastFailTst(sortedIndex, true);
+    fastFailTst(ssSortedIndex, false);
+    
+    
 
+    // Test find()
+    setupWords();
+    
+    tstWord(wordSetIndex);
+    tstWord(ssWordSetIndex);
+    
   }
   
-  private void setIteratorWithoutMods(FSIndex<FeatureStructure> setIndex, int thrd) {
+  private void setIteratorWithoutMods(FSIndex<FeatureStructure> setIndex, int threadNumber) {
     // /////////////////////////////////////////////////////////////////////////
     // Create a reverse iterator for the set index and check that the result
     // is the same as for forward iteration.
@@ -482,12 +482,17 @@ public class IteratorTest extends TestCase {
     FSIterator<FeatureStructure> it = setIndex.iterator();
     AnnotationFS a, b = null;
     int ii = 0;
-    StringBuilder sb = new StringBuilder();
+//    StringBuilder sb = new StringBuilder();
     while (it.isValid()) {
       a = (AnnotationFS) it.get();
       if (b != null) {
-        // note compare may be equal for two items of different types 
-        assertTrue(setIndex.compare(b, a) <= 0);
+        // note compare may be equal for two items of different types
+        // because setIndex is not using type priorities (I think) (2/2015)
+        int compareResult = setIndex.compare(b, a);
+        if (compareResult == 0) {
+          // types must be different
+          assertFalse(a.getType().getName().equals(b.getType().getName()));
+        }
         if (a.hashCode() == b.hashCode()) {
           System.err.format("set Iterator: should not have 2 identical elements%n%s%n", it);
           assertTrue(false);
@@ -498,20 +503,20 @@ public class IteratorTest extends TestCase {
 //        }
       }
       b = a;
-       sb.append(String.format("%d %d debug: n=%d, type=%s, start=%d, end-%d%n",
-           thrd,
-           ii++,
-           a.hashCode(),
-           a.getType().getName(),
-           a.getBegin(),
-           a.getEnd()));
+//       sb.append(String.format("%d %d debug: n=%d, type=%s, start=%d, end-%d%n",
+//           threadNumber,
+//           ii++,
+//           a.hashCode(),
+//           a.getType().getName(),
+//           a.getBegin(),
+//           a.getEnd()));
       v.add(it.get().hashCode());
       it.moveToNext();
     }
     // System.out.println("Number of annotations: " + v.size());
     if (v.size() != ((10 * 3) + (10 * 3))) {
       System.err.format("Expected number in set was 60, but has %d elements%n%s%n", v.size(), it);
-      System.err.println(sb);
+//      System.err.println(sb);
       assertTrue(false);
     }
 //    else
@@ -562,6 +567,108 @@ public class IteratorTest extends TestCase {
     }
   }
   
+  private void findTst(FSIndex index, FSIndex jcasIndex) {
+    findTestCas(index);
+    findTestJCas(jcasIndex);
+  }
+  
+  private void basicRemoveAdd(FSIndex<FeatureStructure> index, int begin, int end) {
+    FSIterator<FeatureStructure> it = index.iterator();
+    it.moveToLast();
+    AnnotationFS a = (AnnotationFS) it.get();
+    cas.removeFsFromIndexes(a);
+    cas.addFsToIndexes(a);
+
+    it.moveToLast();
+    a = (AnnotationFS) it.get();
+    assertEquals(tokenType, a.getType());
+    assertEquals(begin, a.getBegin());
+    assertEquals(end, a.getEnd());
+  }
+  
+  private void fastFailTst(FSIndex<FeatureStructure> index, boolean isShouldFail) {
+    FSIterator<FeatureStructure> it = index.iterator();
+    it.moveToLast();
+    it.moveToFirst();
+    // moved to first, 2.7.0, because new bag iterator is more forgiving re concurrentmodexception
+    FeatureStructure a = it.get();
+    
+    cas.removeFsFromIndexes(a);
+    cas.addFsToIndexes(a);    
+    
+    expectCCE(a, it, isShouldFail);
+    expectCCE(a, it, false);  // ok because above expectCCE reset the iterator   
+  }
+  
+  private void expectCCE(FeatureStructure a, FSIterator it, boolean isShouldFail) {
+    boolean ok = false;
+    try {
+      it.moveToNext();
+    } catch (ConcurrentModificationException e) {
+      ok = true;
+    }
+    assertTrue(isShouldFail ? ok : !ok);
+    
+    it.moveTo(a);  // should reset concurrent mod, 
+    ok = true;
+    try {
+      it.moveToNext();
+    } catch (ConcurrentModificationException e) {
+      ok = false;
+    }
+//    if (!ok) {
+//      System.out.println("debug");
+//    }
+    assertTrue(ok);
+  }
+  
+  private void setupWords() {
+    wType = this.cas.getTypeSystem().getType("org.apache.uima.cas.test.types.Word");
+
+    wordFeat = wType.getFeatureByBaseName("word");
+
+    for (int i = 0; i < 20; i++) {
+      FeatureStructure fs = this.cas.createFS(wType);
+      fs.setStringValue(wordFeat, "word" + i);
+      this.cas.getIndexRepository().addFS(fs);
+    }
+
+    wordSetIndex = this.cas.getIndexRepository().getIndex("Word Set Index");
+    ssWordSetIndex = wordSetIndex.withSnapshotIterators();
+
+  }
+  
+  private void tstWord(FSIndex<FeatureStructure> index) {
+    FSIterator<FeatureStructure> it = index.iterator();
+    it.moveToLast();
+
+    FeatureStructure fs = this.cas.createFS(wType);
+    fs.setStringValue(wordFeat, "word1");
+
+    // TEST moveTo() and get()
+    it.moveTo(fs);
+
+    assertSame(fs.getType(), it.get().getType());
+
+    Type t1 = fs.getType();
+    Type t2 = wordSetIndex.find(fs).getType();
+    assertSame(t1, t2);
+
+  }
+  
+  private void findTestCas(FSIndex<FeatureStructure> index) {
+    AnnotationFS annot = (AnnotationFS) index.iterator().get();  // first element
+    assertNotNull(index.find(annot));
+    assertNull(index.find(this.cas.createAnnotation(this.annotationType, -1, -1)));
+  }
+  
+  private void findTestJCas(FSIndex<?> index) {
+    Annotation annot = (Annotation) index.iterator().get();  // first element
+    assertNotNull(index.find(annot));
+    assertNull(index.find(this.cas.createAnnotation(this.annotationType, -1, -1)));    
+  }
+  
+  
   private void sortedIteratorWithoutMods(FSIndex<FeatureStructure> sortedIndex) {
     // /////////////////////////////////////////////////////////////////////////
     // Test sorted index.
@@ -609,8 +716,8 @@ public class IteratorTest extends TestCase {
       // System.out.println("Iteration: " + i);
       it2.moveToFirst();
       it2.moveTo(list.get(i));
-      assertTrue(((AnnotationFS) it2.get()).getBegin() == ((AnnotationFS) list.get(i)).getBegin());
-      assertTrue(((AnnotationFS) it2.get()).getEnd() == ((AnnotationFS) list.get(i)).getEnd());
+      assertTrue(it2.get().getBegin() == list.get(i).getBegin());
+      assertTrue(it2.get().getEnd() == list.get(i).getEnd());
     }
 
     // Check that reverse iterator produces reverse sequence.
@@ -664,12 +771,21 @@ public class IteratorTest extends TestCase {
     IntVector v = new IntVector();
     FSIterator<FeatureStructure> it = bagIndex.iterator();
     AnnotationFS a, b = null;
-    while (it.isValid()) {
-      a = (AnnotationFS) it.get();
-      assertTrue(a != null);
-      if (b != null) {
-        assertTrue(bagIndex.compare(b, a) <= 0);
+//    int debug_i = 0;
+    while (true) {
+//      if (debug_i == 20) {
+//        System.out.println("Debug");
+//      }
+      if (!it.isValid()) {
+        break;
       }
+       a = (AnnotationFS) it.get();
+//      debug_i ++;
+      assertTrue(a != null);
+      // bag indices no longer are in sort by fs order
+//      if (b != null) {
+//        assertTrue(bagIndex.compare(b, a) <= 0);
+//      }
       b = a;
       v.add(a.hashCode());
       it.moveToNext();
@@ -679,6 +795,9 @@ public class IteratorTest extends TestCase {
     // Check that reverse iterator produces reverse sequence.
     it.moveToLast();
     for (int i = v.size() - 1; i >= 0; i--) {
+//      if (!it.isValid()) {
+//        System.out.println("debug");
+//      }
       assertTrue(it.isValid());
       assertTrue(it.get().hashCode() == v.get(i));
       it.moveToPrevious();
@@ -699,6 +818,9 @@ public class IteratorTest extends TestCase {
       it.moveToNext();
       assertTrue(it.isValid());
       assertTrue(it.get().hashCode() == v.get(current + 1));
+//      if (current == 19) {
+//        System.out.println("debug");
+//      }
       it.moveToPrevious();
       assertTrue(it.isValid());
       assertTrue(it.get().hashCode() == v.get(current));
@@ -950,6 +1072,9 @@ public class IteratorTest extends TestCase {
     } catch (Exception e) {
       caught = true;
     }
+//    if (caught != true) {
+//      System.out.println("Debug");
+//    }
     assertTrue(caught);
   }
   
